unit DebugU;

interface

uses SysUtils,Classes,Forms,AlfaBetaU;

type
  TDebugLevel = (dlNone,dlNormal,dlExtra);

  TDebug = class(TAbstractDebug)
  private
    { debug su file }
    TxtFileName: string;
    TxtFile: TextFile;
    OutputToFile: boolean;
    { debug su TStringList }
    StrList: TStrings;
    OutputToList: boolean;
  protected
    Level: TDebugLevel; // una volta impostato non va pi cambiato
    procedure Initialize(DL: TDebugLevel);
    function canWrite(DM: TDebugMode): boolean;
  public
    constructor Create(DL: TDebugLevel; List: TStrings);
    constructor CreateFile(DL: TDebugLevel; FileName: string = 'debug.txt');
    procedure Open; override;
    procedure Write(DM: TDebugMode; FormatString: string;
                     Args: array of const); override;
    procedure Close; override;
  end;

implementation

{ TDebug }

constructor TDebug.Create(DL: TDebugLevel; List: TStrings);
begin
  StrList := List;
  OutputToList := true;
  Initialize(DL);
end;

constructor TDebug.CreateFile(DL: TDebugLevel; FileName: string);
begin
  TxtFileName := FileName;
  OutputToFile := true;
  Initialize(DL);
end;

procedure TDebug.Initialize(DL: TDebugLevel);
begin
  Level := DL;
end;

procedure TDebug.Open;
begin
  if (Level <> dlNone) then
  begin
    if OutputToFile then
    begin
      AssignFile(TxtFile,TxtFilename);
      Rewrite(TxtFile)
    end;
    if OutputToList then StrList.Clear;
  end
end;

procedure TDebug.Close;
begin
  if (Level <> dlNone) and OutputToFile then
    CloseFile(TxtFile)
end;

procedure TDebug.Write(DM: TDebugMode; FormatString: string;
  Args: array of const);
var
  OutStr: string;
begin
  if canWrite(DM) then
  begin
    OutStr := Format(FormatString,Args);
    if OutputToFile then writeln(TxtFile,OutStr);  // flush?
    if OutputToList then
    begin
      StrList.Add(OutStr);
      // forse questo andrebbe fatto con una callback function
      Application.ProcessMessages
    end
  end
end;

function TDebug.canWrite(DM: TDebugMode): boolean;
begin
  result := ((Level = dlNormal) and (DM = dmNormal)) or
            ((Level = dlExtra) and (DM = dmNormal)) or
            ((Level = dlExtra) and (DM = dmExtra)) 
end;

end.
