unit Link;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, DdeMan;

type
  TLinkType = (ltInternal,ltDDE);
  TAlgorithm = (alMinimax,alAlfabeta);

  TLinkFrm = class(TForm)
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    DdeClientConv1: TDdeClientConv;
    DdeClientItem1: TDdeClientItem;
    OpenDialog1: TOpenDialog;
    OpzioniBox: TGroupBox;
    MinimaxRadio: TRadioButton;
    AlfaBetaRadio: TRadioButton;
    DepthEdt: TLabeledEdit;
    TimeChk: TCheckBox;
    LinkBox: TGroupBox;
    InternalEngineRadio: TRadioButton;
    DDELinkRadio: TRadioButton;
    DDEPnl: TPanel;
    ServiceBtn: TSpeedButton;
    ServiceLbl: TLabel;
    ConnectBtn: TButton;
    ServiceCombo: TComboBox;
    procedure InternalEngineRadioClick(Sender: TObject);
    procedure DDELinkRadioClick(Sender: TObject);
    procedure ConnectBtnClick(Sender: TObject);
    procedure ServiceBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OkBtnClick(Sender: TObject);
  private
    { Private declarations }
    procedure AggiornaCombo(combo: TComboBox);
  public
    { Public declarations }
    LinkType: TLinkType;
    DDEService: string;
    Algorithm: TAlgorithm;
    AlgoDepth: integer;
    RequireTime: boolean;
    function DDETopic: string;
    function Connetti(flag: boolean): boolean;
  end;

var
  LinkFrm: TLinkFrm;

implementation

{$R *.dfm}

procedure TLinkFrm.AggiornaCombo(combo: TComboBox);
var
  idx: integer;
  count: integer;
begin
  idx := combo.Items.IndexOf(combo.text);
  if idx = -1 then
    combo.Items.Insert(0,combo.text)
  else
    combo.Items.Move(idx,0);

  count := combo.DropDownCount;
  while (count > 0) and (combo.Items.Count > count) do
    combo.Items.Delete(count);
end;

procedure TLinkFrm.InternalEngineRadioClick(Sender: TObject);
begin
  Height := 182;
  DDEPnl.Visible := false
end;

procedure TLinkFrm.DDELinkRadioClick(Sender: TObject);
begin
  Height := 246;
  DDEPnl.Visible := true
end;

procedure TLinkFrm.ConnectBtnClick(Sender: TObject);
begin
  if (ServiceCombo.Text <> '') and Connetti(false) then
  begin
    //DDEClientItem1.DdeItem := '         ';
    ShowMessage('Connessione stabilita con successo');
    AggiornaCombo(ServiceCombo);
    DDEClientConv1.CloseLink;
  end
  else ShowMessage('Connessione fallita')
end;

procedure TLinkFrm.ServiceBtnClick(Sender: TObject);
var
  ServiceDir,ServiceName: string;
  idx: integer;
begin
  if ServiceCombo.Text <> '' then
    OpenDialog1.FileName := ServiceCombo.Text;

  if OpenDialog1.Execute then
  begin
    ServiceName := ExtractFileName(OpenDialog1.FileName);
    { tolgo l'estensione }
    idx := LastDelimiter('.',ServiceName);
    if idx>0 then ServiceName := Copy(ServiceName,1,idx-1);
    { ora aggiungo eventualmente il percorso }
    ServiceDir := ExtractFilePath(OpenDialog1.FileName);
    if ExtractFilePath(Application.ExeName) <> ServiceDir then
      ServiceName := ServiceDir + ServiceName;
    { aggiorno il combo }
    ServiceCombo.Text := ServiceName;
    AggiornaCombo(ServiceCombo);
  end
end;

procedure TLinkFrm.FormCreate(Sender: TObject);
begin
  LinkType := ltInternal;
  DDEService := 'prologFiletto';
  Algorithm := alAlfabeta;
  AlgoDepth := 9;
  RequireTime := true;
end;

procedure TLinkFrm.FormShow(Sender: TObject);
begin
  case LinkType of
  ltInternal: InternalEngineRadio.Checked := true;
  ltDDE: DDELinkRadio.Checked := true
  end;
  ServiceCombo.Text := DDEService;
  case Algorithm of
  alMinimax: MinimaxRadio.Checked := true;
  alAlfabeta: AlfabetaRadio.Checked := true
  end;
  DepthEdt.Text := IntToStr(AlgoDepth);
  TimeChk.Checked := RequireTime;
end;

function TLinkFrm.DDETopic: string;
begin
  case Algorithm of
  alMinimax: result := 'minimax';
  alAlfabeta: result := 'alfabeta';
  end;
  result := result + ',' + IntToStr(AlgoDepth);
  if requireTime then
    result := result + ',T';
end;

procedure TLinkFrm.OkBtnClick(Sender: TObject);
begin
  if InternalEngineRadio.Checked then
    LinkType := ltInternal
  else LinkType := ltDDE;
  DDEService := ServiceCombo.Text;
  if MinimaxRadio.Checked then
    Algorithm := alMinimax
  else Algorithm := alAlfabeta;
  AlgoDepth := StrToInt(DepthEdt.Text);
  RequireTime := TimeChk.Checked
end;

{
Connessione al server DDE
 flag = true se richiamato dall'interno
 flag = false se  richiamato dall'esterno
}
function TLinkFrm.Connetti(flag: boolean): boolean;
begin
  if (flag and (LinkType = ltDDE)) or
     (not flag and DDELinkRadio.Checked) then
    result :=
      DDEClientConv1.SetLink(ServiceCombo.Text,DDETopic) or
      { secondo tentativo }
      DDEClientConv1.SetLink(ServiceCombo.Text,DDETopic)
  else result := true;
end;

end.
