unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ToolWin, ExtCtrls, ImgList, DdeMan;

type
  TPlayer = (human,computer);
  TStatus = (gsInizio,gsHuman,gsComputer,gsPatta,gsVittoria);

  TMainFrm = class(TForm)
    StatusBar: TStatusBar;
    ToolBar: TToolBar;
    NewGameBtn: TToolButton;
    LinkBtn: TToolButton;
    ChbImg: TImage;
    Sep1: TToolButton;
    Sep2: TToolButton;
    AboutBtn: TToolButton;
    ImageList1: TImageList;
    DebugBtn: TToolButton;
    ToolButton2: TToolButton;
    procedure FormResize(Sender: TObject);
    procedure NewGameBtnClick(Sender: TObject);
    procedure AboutBtnClick(Sender: TObject);
    procedure ChbImgMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure LinkBtnClick(Sender: TObject);
    procedure DebugBtnClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    LeftB,TopB,SqSide: integer;
    TempoGiocatore: TDateTime;
    Giocatore: array[1..2] of TPlayer;
    Ply: 0..9;
    GameRun: TStatus;
    procedure Disegna;
    procedure AggiornaStatusBar;
    function ConnessioneDDE(ClientItem: TDDEClientItem;
      Service,Topic,Item: string): string;
    procedure GiocaUmano(X, Y: integer);
    function FaiMossa(CellX,CellY: integer): integer;
    function isGameOver(var Status: TStatus): boolean;
    procedure ProxFase;
    function GiocaComputer(var cellX,cellY: integer; var Tempo: string): boolean;
  public
    { Public declarations }
    Board: array[1..9] of char; { pubblico per scopi di debug }
    procedure IniziaGioco(GameType: integer);
  end;

var
  MainFrm: TMainFrm;

implementation

uses Math, NewGame, Link, ParseStringU, InternalEngine, MvList, DebugView;

{$R *.dfm}

{ Funzioni grafiche }

procedure TMainFrm.Disegna;
var
  row,col,idx: integer;
  w,h: integer;
  SqX,SqY: integer;
  TextX,TextY: integer;
  FontSize: integer;
  TextSz: TSize;
begin
  if GameRun <> gsInizio then
  begin
    w := ChbImg.Width;
    h := ChbImg.Height;
    if w < h then
    begin
      SqSide := w div 3;
      LeftB := w - 3*SqSide;
      TopB := (h - 3*SqSide) div 2;
    end
    else
    begin
      SqSide := h div 3;
      LeftB := (w - 3*SqSide) div 2;
      TopB := h - 3*SqSide;
    end;

    FontSize := SqSide div 2;
    with ChbImg.Canvas do
    begin
      Font.Name := 'Times New Roman';
      Font.Size := FontSize;
      FillRect(Rect(0,0,w,h));
      TextSz := TextExtent('X')
    end;
    TextX := (SqSide-TextSz.cx) div 2;
    TextY := (SqSide-TextSz.cy) div 2;

    for row := 0 to 2 do for col := 0 to 2 do
    begin
      SqX := LeftB + col*SqSide;
      SqY := TopB + row*SqSide;
      idx := 1 + col + row*3;
      ChbImg.Canvas.Rectangle(SqX, SqY, SqX+SqSide, SqY+SqSide);
      ChbImg.Canvas.TextOut(SqX+TextX,SqY+TextY,Board[Idx]);
    end
  end
end;

procedure TMainFrm.AggiornaStatusBar;
var
  s: string;
begin
  case GameRun of
  gsHuman,gsComputer: s := 'Tocca al giocatore ' + IntToStr((Ply mod 2) + 1);
  gsPatta:     s := 'Partita patta';
  gsVittoria:  s := 'Ha vinto il giocatore ' + IntToStr((Ply mod 2) + 1);
  else raise Exception.Create('Errore');
  end;
  StatusBar.SimpleText := s;
end;

{ Funzioni di gioco }

procedure TMainFrm.IniziaGioco(GameType: integer);
begin
  if (GameType > 0) and not LinkFrm.Connetti(true) then
  begin
    Application.MessageBox('Connessione al server fallita','Nuova partita',
      MB_ICONERROR);
    Exit
  end;

  case GameType of
  0: begin Giocatore[1] := human;    Giocatore[2] := human    end;
  1: begin Giocatore[1] := human;    Giocatore[2] := computer end;
  2: begin Giocatore[1] := computer; Giocatore[2] := human    end;
  3: begin Giocatore[1] := computer; Giocatore[2] := computer end;
  end;
  if Giocatore[1] = human then
  begin
    TempoGiocatore := Time;
    GameRun := gsHuman
  end
  else GameRun := gsComputer;
  Ply := 0;
  Board := '         ';
  MvListFrm.ListView1.Clear;
  AggiornaStatusBar;
  Disegna;
  ProxFase;
end;

{ richiamata da OnMouseDown se GameRun = gsHuman }
procedure TMainFrm.GiocaUmano(X,Y: integer);
var
  CellX,CellY: integer;
  Mossa: integer;
  Tempo: TDateTime;
begin
  if (X >= LeftB) and (X < LeftB+3*SqSide) and
     (Y >= TopB) and (Y < TopB+3*SqSide) then
  begin
    CellX := (X-LeftB) div SqSide;
    CellY := (Y-TopB) div SqSide;
    Tempo := TempoGiocatore;
    Mossa := FaiMossa(CellX,CellY);
    if Mossa > 0 then
    begin
      Tempo := Time - Tempo;
      MvListFrm.AggiungiRiga(Ply,'H',2-(ply mod 2),mossa,0,
                             IntToStr(Trunc(Tempo*24*3600)) + ' sec.');
    end;
    ProxFase
  end
end;

{ Decide se gioca il computer }
procedure TMainFrm.ProxFase;
var
  cellX,cellY: integer;
  Tempo: string;
begin
  if (GameRun = gsComputer) then
  begin
    if GiocaComputer(cellX,cellY,Tempo) then
    begin
      FaiMossa(cellX,cellY);
      if Tempo <> '' then
        ShowMessage('Tempo impiegato: ' + Tempo + ' millisecondi');
      ProxFase
    end
    else Application.MessageBox('Problemi col server','Gioco del filetto',MB_ICONERROR)
  end
end;

{ fa la mossa del computer }
function TMainFrm.GiocaComputer(var cellX, cellY: integer;
  var Tempo: string): boolean;
var
  value: string;
  mossa,valore: integer;
  params: array[1..3] of string;
  num: integer;
  Save_Cursor:TCursor;
begin
  with LinkFrm do
    if LinkType = ltDDE then
    begin
      Save_Cursor := Screen.Cursor;
      Screen.Cursor := crHourGlass;    { Show hourglass cursor }
      try
        { Do some lengthy operation }
        value := ConnessioneDDE(DDEClientItem1,DDEService,DDETopic,Board);
      finally
        Screen.Cursor := Save_Cursor;  { Always restore to normal }
      end;
      num := ParseString(value,',',params);
//      ShowMessage(params[1]+','+params[2]+','+params[3]);
      mossa := StrToInt(params[1]) - 1;
      valore := StrToInt(params[2]);
      if num >= 3 then Tempo := params[3]
      else Tempo := '';
      result := true
    end
    else
    begin
      Save_Cursor := Screen.Cursor;
      Screen.Cursor := crHourGlass;    { Show hourglass cursor }
      try
        { Do some lengthy operation }
        MossaComputer(DDETopic,Board,mossa,valore,tempo);
        Dec(mossa); // mossa := 0..8
      finally
        Screen.Cursor := Save_Cursor;  { Always restore to normal }
      end;
      result := true
    end;

  if result then
  begin
    MvListFrm.AggiungiRiga(Ply+1,'C',(Ply mod 2)+1,
                           mossa+1,valore,tempo + ' millisec.');
    cellY := mossa div 3;
    cellX := mossa mod 3;
  end
end;

{ richiamata per fare la mossa sia dei giocatori sia dell'AI }
function TMainFrm.FaiMossa(CellX,CellY: integer): integer;
var
  Pezzo: char;
  idx: integer;
begin
  idx := CellX + 1 + CellY*3;
  if Board[idx] = ' ' then
  begin
    if (Ply mod 2) = 0 then Pezzo := 'X'
    else Pezzo := 'O';
    Board[idx] := Pezzo;
    Inc(Ply);
    if not isGameOver(GameRun) then
    begin
      if Giocatore[(ply mod 2) + 1] = human then
      begin
        TempoGiocatore := Time;
        GameRun := gsHuman
      end
      else GameRun := gsComputer;
    end;
    AggiornaStatusBar;
    Disegna;
    result := idx
  end
  else result := 0;
end;

{ testa se  finita la partita e nel caso ritorna lo stato appropriato
  Se la partita non  finita ritorna false }
function TMainFrm.isGameOver(var Status: TStatus): boolean;
  function filetto(i1,i2,i3: integer): boolean;
  begin
    result := (board[i1] <> ' ') and (board[i1] = board[i2]) and
              (board[i2] = board[i3])
  end;
  function isPatta: boolean;
  var i: integer;
  begin
    result := true;
    for i := 1 to 9 do if board[i] = ' ' then
    begin
      result := false;
      break
    end
  end;
begin
  if filetto(1,2,3) or filetto(4,5,6) or filetto(7,8,9) or
     filetto(1,4,7) or filetto(2,5,8) or filetto(3,6,9) or
     filetto(1,5,9) or filetto(3,5,7) then
  begin
    Status := gsVittoria;
    result := true
  end
  else if isPatta then { deve stare dopo }
  begin
    Status := gsPatta;
    result := true
  end
  else result := false
end;

{ Collegamento al motore esterno }

function TMainFrm.ConnessioneDDE(ClientItem: TDDEClientItem;
      Service,Topic,Item: string): string;
var
  p: PChar;
  ClientConv: TDDEClientConv;
begin
  ClientConv := ClientItem.DdeConv;
  { Attenzione! se il server non  aperto o non  nella stessa
    cartella fallisce }
  if ClientConv.SetLink(Service,Topic) then
    try
      ClientItem.DdeItem := Item;
      p := ClientConv.RequestData(Item);
      if assigned(p) then
      begin
        result := p;
        StrDispose(p)
      end
      else raise Exception.Create('Errore nella ricezione dei dati');
    finally
      ClientConv.CloseLink;
    end
  else raise Exception.Create('Connessione al motore fallita');
end;

{ Interfaccia }

procedure TMainFrm.FormResize(Sender: TObject);
begin
  if GameRun <> gsInizio then
  begin
    with ChbImg do
    begin
      Picture.BitMap.Width := Width;
      Picture.Bitmap.Height := Height
    end;
    Disegna
  end
end;

procedure TMainFrm.NewGameBtnClick(Sender: TObject);
begin
  if NewGameFrm.ShowModal = mrOk then
  begin
    IniziaGioco(NewGameFrm.GameTypeRadio.ItemIndex);
    Disegna
  end
end;

procedure TMainFrm.AboutBtnClick(Sender: TObject);
var
  autori: string;
  DateLimit: TDateTime;
begin
  { data limite dell'esame ;-) }
  DateLimit := EncodeDate(2003,1,1);
  if Date > DateLimit then
    autori := 'scritto da F. Bisi'
  else
    autori := 'scritto da F. Bisi, P. Cristofanelli, P. Di Tomasso';
  ShowMessage('Gioco del filetto'#10#13 + autori);
end;

procedure TMainFrm.ChbImgMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if GameRun = gsHuman then GiocaUmano(X,Y);
end;

procedure TMainFrm.LinkBtnClick(Sender: TObject);
begin
  LinkFrm.ShowModal
end;

procedure TMainFrm.DebugBtnClick(Sender: TObject);
begin
  if (Sender as TToolButton).Down then MvListFrm.Show
  else MvListFrm.Close;
end;

procedure TMainFrm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ssCtrl in Shift then
  begin
    { combinazioni di tasti per il debug }
    { copia la partita in corso nella stringa di debug }
    if Key = VK_F11 then
    begin
      DebugFrm.ChbEdit.Text := Board;
      Key := 0
    end
    { apre la finestra di debug }
    else if Key = VK_F12 then
    begin
      DebugFrm.Show;
      Key := 0
    end
  end
end;

end.
