unit DebugView;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Mask, ComCtrls, ParseStringU, DebugU,
  AlfaBetaU;

type
  TDebugFrm = class(TForm)
    DbgList: TListBox;
    TopPnl: TPanel;
    Bianco3Btn: TButton;
    ChbLbl: TLabel;
    ResultBar: TStatusBar;
    OpzioniBox: TGroupBox;
    MinimaxRadio: TRadioButton;
    AlfaBetaRadio: TRadioButton;
    TimeChk: TCheckBox;
    DbgLevelRadio: TRadioGroup;
    ChbEdit: TEdit;
    AnswerPnl: TPanel;
    AnswerLbl: TLabel;
    AnswerEdt: TEdit;
    SendAnswerBtn: TButton;
    DetailsChk: TCheckBox;
    MosseLbl: TLabel;
    MosseEdt: TEdit;
    MosseBtn: TButton;
    WhtRadio: TRadioButton;
    BlkRadio: TRadioButton;
    WhiteMovesBtn: TButton;
    BlackMovesBtn: TButton;
    CheckBtn: TButton;
    Nero2Btn: TButton;
    Bianco1Btn: TButton;
    ResultRadio: TRadioGroup;
    procedure ChbEditKeyPress(Sender: TObject; var Key: Char);
    procedure Bianco3BtnClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SendAnswerBtnClick(Sender: TObject);
    procedure MosseBtnClick(Sender: TObject);
    procedure MosseEdtKeyPress(Sender: TObject; var Key: Char);
    procedure AnswerEdtKeyPress(Sender: TObject; var Key: Char);
    procedure WhiteMovesBtnClick(Sender: TObject);
    procedure BlackMovesBtnClick(Sender: TObject);
    procedure CheckBtnClick(Sender: TObject);
    procedure Nero2BtnClick(Sender: TObject);
    procedure Bianco1BtnClick(Sender: TObject);
  private
    { Private declarations }
    procedure MosseDaPos(Position: TAbstractPosition; var Moves: TMovesList);
    procedure VisualizzaMosse(Moves: TMovesList; List: TStrings);
    procedure VisualizzaMosseValori(Moves: TMovesList; Values: TValuesList;
      List: TStrings);
    function Controlla(Giocatore, Board: string; controllo: integer): string;
    procedure ChiamaAlgoritmo(Giocatore: string; Depth: integer);
    function FaiDebug(Topic,Board,Giocatore: string; Depth: integer;
      DbgLevel: TDebugLevel = dlNone; StrList: TStrings = nil): string;
  public
    { Public declarations }
    procedure GeneraMosse(Giocatore: string; Board: string; ListBox: TListBox);
  end;

var
  DebugFrm: TDebugFrm;

implementation

uses Main, InternalEngine, Game, MoveU, PositionU, ModuloDati;

{$R *.dfm}

procedure TDebugFrm.ChbEditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
  begin
    Bianco3BtnClick(Sender);
    Key := #0
  end
  else if (Key <> 'O') and (Key <> 'X') and (Key <> ' ') then
  begin
    Beep;
    Key := #0
  end
end;

procedure TDebugFrm.ChiamaAlgoritmo(Giocatore: string; Depth: integer);
var
  value: string;
  DbgLevel: TDebugLevel;
  Save_Cursor:TCursor;

  {  la DDETopic ricopiata }
  function BuildTopicString: string;
  begin
    if MinimaxRadio.Checked then result := 'minimax'
    else result := 'alfabeta';
    if TimeChk.Checked then
      result := result + ',y'
    else result := result + ',n';
    if DetailsChk.Checked then
      result := result + ',y'
    else result := result + ',n';
  end;
begin
  DbgLevel := TDebugLevel(DbgLevelRadio.ItemIndex);
  DbgList.Clear;
  ResultBar.SimpleText := 'Elaboro...';
  Save_Cursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;    { Show hourglass cursor }
  try
    { Do some lengthy operation }
    DbgList.Columns := 0;
    value := FaiDebug(BuildTopicString,ChbEdit.Text,Giocatore,Depth,
                  DbgLevel,DbgList.Items);
  finally
    Screen.Cursor := Save_Cursor;  { Always restore to normal }
  end;
  ResultBar.SimpleText := value;
  //AnswerEdt.Text := value;
end;

procedure TDebugFrm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if MainFrm.AmministraTastiDebug(Key,Shift) then Key := 0
end;

procedure TDebugFrm.SendAnswerBtnClick(Sender: TObject);
begin
  MainFrm.ProcessaRisposta(AnswerEdt.Text)
end;

procedure TDebugFrm.MosseBtnClick(Sender: TObject);
var
  params: TStringArray;
  i: integer;
  pl: TPlayer;
begin
  params := ParseString(MosseEdt.Text,',');
  try
    if WhtRadio.Checked then pl := cplWhite
    else pl := cplBlack;
    for i := Low(params) to High(params) do
    begin
      FaiMossa(pl,Params[i]);
      if pl = cplBlack then pl := cplWhite
      else pl := cplBlack;
    end;
    ChbEdit.Text := BoardString;
    MainFrm.AggiornaGrafica;
  finally
    params := nil
  end
end;

procedure TDebugFrm.MosseEdtKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
  begin
    Key := #0;
    MosseBtnClick(sender);
  end
end;

procedure TDebugFrm.AnswerEdtKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
  begin
    Key := #0;
    SendAnswerBtnClick(sender);
  end
end;

function TDebugFrm.FaiDebug(Topic,Board,Giocatore: string; Depth: integer;
  DbgLevel: TDebugLevel = dlNone; StrList: TStrings = nil): string;

  function AnswerToBool(answer: string): boolean;
  begin
    result := answer = 'y'
  end;
var
  Iniz,Finale: TDateTime;
  num: integer;
  params: TStringArray;
  Position: TPosition;
  Algo: TAlgorithm;
  Debug: TDebug;
  Valore: integer;
  Mossa,Tempo: string;
  RequireTime,RequireDetails: boolean;
  CodErrore: string;
  success: boolean;
  Mosse: TMovesList;
  Valori: TValuesList;
  BestMove: TAbstractMove;
  Mosse1: TMovesList;
  i: integer;
begin
  { prima fase: lettura del topic }
  params := ParseString(Topic,',');
  // per un warning idiota devo inizializzarmi RT, RD e Algo
  RequireTime := false; RequireDetails := false; Algo := alMinimax;
  try
    num := Length(params);
    if num <> 3 then
      raise Exception.Create('topic malformato');
    if params[0] = 'minimax' then
      Algo := alMinimax
    else if params[0] = 'alfabeta' then
      Algo := alAlfabeta
    else
      raise Exception.Create('Algoritmo sconosciuto');
    RequireTime := AnswerToBool(params[1]);
    RequireDetails := AnswerToBool(params[2]);
  finally
    params := nil
  end;

  { seconda fase: chiamata dell'algoritmo }
  //  Debug := nil; // trucco cos posso usare un solo try
  Position := TPosition.Create;
  BestMove := Position.CreateMove;
  if assigned(StrList) then
    Debug := TDebug.Create(DbgLevel,StrList)
  else
    Debug := TDebug.CreateFile(DbgLevel,'debug.txt');
  try
    Position.SetPosition(Giocatore,Board);
    Iniz := Time;
    if ResultRadio.ItemIndex = 0 then
    begin
      success := ChooseMove(Algo,Debug,Depth,Position,BestMove,Valore);
      Mossa := BestMove.AsString;
      if RequireDetails then
      begin
        Position.DoMove(BestMove);
        CalcolaDettagli(Algo,Debug,2,Position,Mosse1);
      end;
    end
    else
    begin
      success := getAllMoves(Algo,Debug,Depth,Position,Mosse,Valori);
      DbgList.Columns := 0;
      VisualizzaMosseValori(Mosse,Valori,DbgList.Items);
      Mosse := nil; Valori := nil
    end;
    Finale := Time;
  finally
    Debug.Free;
    BestMove.Free;
    Position.Free
  end;

  { terza fase: output }
  if success then CodErrore := 'ok'
  else CodErrore := 'illegal position';
  result := CodErrore;
  if success then result := result + ',' + Mossa + ',' + IntToStr(Valore);
  if RequireTime then
  begin
    Tempo := IntToStr(Trunc((Finale-Iniz)*24*3600*1000));
    result := result + ',' + Tempo;
  end;
  if RequireDetails then
  begin
    for i := Low(Mosse1) to High(Mosse1) do
    begin
      result := result + ',' + Mosse1[i];
    end
  end
end;

procedure TDebugFrm.VisualizzaMosseValori(Moves: TMovesList; Values: TValuesList;
  List: TStrings);
var
  i: integer;
begin
  List.Clear;
  for i := Low(Moves) to High(Moves) do
  begin
    List.Add(Moves[i] + ',' + IntToStr(Values[i]))
  end
end;

procedure TDebugFrm.VisualizzaMosse(Moves: TMovesList; List: TStrings);
var
  i: integer;
begin
  List.Clear;
  for i := Low(Moves) to High(Moves) do
  begin
    List.Add(Moves[i])
  end
end;

procedure TDebugFrm.GeneraMosse(Giocatore: string; Board: string;
       ListBox: TListBox);
var
  Position: TPosition;
  Mosse: TMovesList;
begin
  Position := TPosition.Create;
  try
    Position.SetPosition(Giocatore,Board);
    MosseDaPos(Position,Mosse);
    try
      ListBox.Columns := 4;
      VisualizzaMosse(Mosse,ListBox.Items)
    finally
      Mosse := nil;
    end
  finally
    Position.Free
  end;
end;

procedure TDebugFrm.MosseDaPos(Position: TAbstractPosition; var Moves: TMovesList);
var
  CurrentMove: TAbstractMove;
  Cnt: integer;
begin
  Moves := nil;
  Cnt := 0;
  CurrentMove := TPosition(Position).CreateMove;
  try
    CurrentMove.First(Position);
    if CurrentMove.isDefined then
      repeat
//        Position.DoMove(CurrentMove);
        SetLength(Moves,Cnt+1);
        Moves[Cnt] := CurrentMove.AsString;
        Inc(Cnt);
//        Position.UndoMove(CurrentMove);
      until not CurrentMove.Next(Position);
  finally
    CurrentMove.Free
  end
end;

function TDebugFrm.Controlla(Giocatore, Board: string;
                              controllo: integer): string;
var
  Position: TPosition;
  Valore: integer;
begin
  Position := TPosition.Create;
  try
    Position.SetPosition(Giocatore,Board);
    case controllo of
    1: // isLegal
    begin
      if Position.isLegal then result := 'si'
      else result := 'no'
    end;
    2: // scacco
    begin
      if Position.ScaccoRe(Position.Giocatore) then result := 'si'
      else result := 'no'
    end;
    3: // matto
    begin
      if Position.isLeaf(Valore) then
      begin
        if Valore = 1 then result := 'si'
        else result := 'patta'
      end
      else result := 'no';
    end;
    else raise Exception.Create('sconosciuto');
    end;
  finally
    Position.Free
  end;
end;

procedure TDebugFrm.WhiteMovesBtnClick(Sender: TObject);
begin
  GeneraMosse('bianco',ChbEdit.Text,DbgList);
end;

procedure TDebugFrm.BlackMovesBtnClick(Sender: TObject);
begin
  GeneraMosse('nero',ChbEdit.Text,DbgList);
end;

procedure TDebugFrm.CheckBtnClick(Sender: TObject);
const
  Messaggi: array[1..5] of string =
  (
    'Scacchiera legale:                       ',
    'Il bianco ha messo sotto scacco il nero: ',
    'E'' scacco matto a favore del bianco:     ',
    'Il nero ha messo sotto scacco il bianco: ',
    'E'' scacco matto a favore del nero:       '
  );
begin
  DbgList.Columns := 0;
  DbgList.Clear;
  DbgList.Items.Add(Messaggi[1] + Controlla('bianco',ChbEdit.Text,1));
  DbgList.Items.Add(Messaggi[2] + Controlla('bianco',ChbEdit.Text,2));
  DbgList.Items.Add(Messaggi[3] + Controlla('nero',ChbEdit.Text,3));
  DbgList.Items.Add(Messaggi[4] + Controlla('nero',ChbEdit.Text,2));
  DbgList.Items.Add(Messaggi[5] + Controlla('bianco',ChbEdit.Text,3));
end;

procedure TDebugFrm.Bianco3BtnClick(Sender: TObject);
begin
  ChiamaAlgoritmo('bianco',3)
end;

procedure TDebugFrm.Nero2BtnClick(Sender: TObject);
begin
  ChiamaAlgoritmo('nero',2)
end;

procedure TDebugFrm.Bianco1BtnClick(Sender: TObject);
begin
  ChiamaAlgoritmo('bianco',1)
end;

end.
