unit Game;

interface

uses SysUtils;

const
  NumSquaresW = 8; // caselle nella scacchiera in orizzontale
  NumSquaresH = 8; // caselle nella sscacchiera in verticale

type
  { TChbPosition - posizione sulla scacchiera (0 = fuori) }
  TChbPosition = 0..NumSquaresW*NumSquaresH; // 0..64
  TPiece = // 0..15
  (
    chpLeftTower,chpLeftKnight,chpLeftBishop,chpKing,
    chpQueen,chpRightTower,chpRightKnight,chpRightBishop,
    chpPawn1,chpPawn2,chpPawn3,chpPawn4,
    chpPawn5,chpPawn6,chpPawn7,chpPawn8
  );
  TPlayer = (cplBlack,cplWhite);
  TPieceType = (cptTower,cptKnight,cptBishop,cptQueen,cptKing,cptPawn);
  TPieces = array[TPlayer,TPiece] of TChbPosition;

const
  { tipi di pezzi non cambiati dalle promozioni }
  OriginalPieceTypes: array[TPlayer,TPiece] of TPieceType =
  (
    (
      cptTower,cptKnight,cptBishop,cptQueen,
      cptKing,cptBishop,cptKnight,cptTower,
      cptPawn,cptPawn,cptPawn,cptPawn,
      cptPawn,cptPawn,cptPawn,cptPawn
    ),
    (
      cptPawn,cptPawn,cptPawn,cptPawn,
      cptPawn,cptPawn,cptPawn,cptPawn,
      cptTower,cptKnight,cptBishop,cptQueen,
      cptKing,cptBishop,cptKnight,cptTower
    )
  );

var
  { pezzi: array[0..1,0..15] of 0..64 }
  Pieces: TPieces;
  { PieceTypes[0..1,0..15] - tipi dei pezzi }
  { non  costante per via delle promozioni dei pedoni }
  PieceTypes: array[TPlayer,TPiece] of TPieceType;

function BoardString: string;
procedure RidisponiPezzi(caso: boolean);
function isEmptyPos(ChbPos: TChbPosition): boolean;
function FindPieceAtPos(ChbPos: TChbPosition;
         var Player: TPlayer; var Piece: TPiece): boolean;
procedure SaveToFile(filename,number,author: string);
procedure SaveToTextFile(filename: string);
function LoadFromFile(filename: string; var number,author: string): boolean;
procedure FaiMossa(Pl: TPlayer; Mossa: string);

implementation

const
  NumLen = 6;
  AuthLen = 40;

procedure SaveToTextFile(filename: string);
var
  TxtFile: TextFile;
  Str: string;
begin
{$I+}
  AssignFile(TxtFile, FileName);
  Rewrite(TxtFile);
  try
    Str := BoardString;
    write(TxtFile,Str)
  finally
    CloseFile(TxtFile);
  end
{$I-}
end;

procedure SaveToFile(filename,number,author: string);
var
  m2mfile: file;
  numberShort: string[NumLen];
  authorShort: string[AuthLen];
begin
{$I+}
  AssignFile(m2mfile, FileName);
  Rewrite(m2mfile, 1);	{ Record size = 1 }
  try
    FillChar(numberShort,NumLen,' '); // per pulire l'output - non servirebbe ma...
    numberShort := number;
    FillChar(AuthorShort,AuthLen,' '); // per pulire l'output - non servirebbe ma...
    authorShort := author;
    BlockWrite(m2mfile, numberShort, NumLen+1);
    BlockWrite(m2mfile, authorShort, AuthLen+1);
    BlockWrite(m2mfile, Pieces, Sizeof(Pieces));
    BlockWrite(m2mfile, PieceTypes, Sizeof(PieceTypes));
  finally
    CloseFile(m2mfile);
  end
{$I-}
end;

function LoadFromFile(filename: string; var number,author: string): boolean;
var
  m2mfile: file;
  numberShort: string[NumLen];
  authorShort: string[AuthLen];
  //PiecesBuf: array[1..16] of byte;
begin
  AssignFile(m2mfile, FileName);
  FileMode := 0;  {Set file access to read only }
  Reset(m2mfile, 1);	{ Record size = 1 }
  if IOResult = 0 then
  begin
    BlockRead(m2mfile, numberShort, NumLen+1);
    BlockRead(m2mfile, authorShort, AuthLen+1);
    BlockRead(m2mfile, Pieces, sizeof(Pieces));
    BlockRead(m2mfile, PieceTypes, Sizeof(PieceTypes));
    number := numberShort;
    author := authorShort;
    CloseFile(m2mfile);
    result := true
  end
  else result := false;
end;

function FindPieceAtPos(ChbPos: TChbPosition;
         var Player: TPlayer; var Piece: TPiece): boolean;
var
  pl: TPlayer;
  i: TPiece;
begin
  result := false;
  for pl := Low(TPlayer) to High(TPlayer) do
    for i := Low(TPiece) to High(TPiece) do
      if Pieces[pl,i] = ChbPos then
      begin
        Player := pl;
        Piece := i;
        result := true;
        break
      end
end;

function isEmptyPos(ChbPos: TChbPosition): boolean;
var
  pl: TPlayer;
  i: TPiece;
begin
  result := not FindPieceAtPos(ChbPos,pl,i);
end;

const
  { questi caratteri definiscono l'interfaccia col motore e
    NON devono essere modificabili dall'utente }
  PieceChars: array[TPlayer,TPieceType] of char =
    (('t','c','a','d','r','p'),
    ('T','C','A','D','R','P'));

function getPieceChar(pl: TPlayer; p: TPiece): char;
begin
  result := PieceChars[pl,PieceTypes[pl,p]]
end;

function getPieceTypeFromChar(ch: char; var t: TPieceType; var pl: TPlayer): boolean;
var
  i: TPlayer;
  j: TPieceType;
begin
  result := false;
  for i := Low(TPlayer) to High(TPlayer) do
    for j := Low(TPieceType) to High(TPieceType) do
      if ch = PieceChars[i,j] then
      begin
        result := true;
        pl := i;
        t := j;
        break
      end
end;

procedure ChbPositionToCoords(val: TChbPosition; var X,Y: integer);
begin
  y := 8 - ((val-1) div 8);
  x := ((val-1) mod 8)+1;
end;

function CoordsToChbPosition(X,Y: integer): TChbPosition;
begin
  result := (8-Y)*8 + X
end;

function BoardString: string;
var
  pl: TPlayer;
  i: TPiece;
  x,y: integer;
  initial: boolean;
begin
  result := '';
  initial := true;
  for pl := Low(TPlayer) to High(TPlayer) do
    for i := Low(TPiece) to High(TPiece) do
      if Pieces[pl,i] <> 0 then
      begin
        ChbPositionToCoords(Pieces[pl,i],X,Y);
        if initial then initial := false
        else result := result + ',';
        result := result + getPieceChar(pl,i) + IntToStr(x) + IntToStr(y);
      end
end;

procedure RidisponiPezzi(caso: boolean);
var
  i: TPiece;
  //{,j}: integer;
  pl: TPlayer;
  val: TChbPosition;
begin
  { inizializza tipi pezzi }
  for pl := Low(TPlayer) to High(TPlayer) do
    for i := Low(Pieces[pl]) to High(Pieces[pl]) do
      PieceTypes[pl,i] := OriginalPieceTypes[pl,i];

  { inizializza pezzi }
  for pl := Low(TPlayer) to High(TPlayer) do
    for i := Low(Pieces[pl]) to High(Pieces[pl]) do
      Pieces[pl,i] := 0; // fuori dalla scacchiera

  if caso then
  begin
    for pl := Low(TPlayer) to High(TPlayer) do
      for i := Low(Pieces[pl]) to High(Pieces[pl]) do
      begin
        repeat val := random(High(TchbPosition)+1) // a caso
        until (val=0) or isEmptyPos(val);
        Pieces[pl,i] := val;
      end
  end
(*
  { ora inizializza scacchiera }
  { prima riga dei pezzi neri }
  for j := 1 to NumSquaresW do
    Chessboard[1,j] := TSquareContent(j);
  { seconda riga dei pezzi neri }
  for j := 1 to NumSquaresW do
    Chessboard[2,j] := TSquareContent(j+NumSquaresW);
  { ora le righe vuote (dalla 3 alla 6) }
  for i := 1 to NumSquaresH do
    for j := 1 to NumSquaresW do
      Chessboard[i,j] := csqNothing;
  { la penultima riga (la settima) con i pedoni bianchi }
  for j := 1 to NumSquaresW do
    Chessboard[NumSquaresH-1,j] := TSquareContent(j+NumSquaresW*2);
  { l'ultima riga con i "pezzi grossi" bianchi }
  for j := 1 to NumSquaresW do
    Chessboard[NumSquaresH,j] := TSquareContent(j+NumSquaresW*3);
*)
end;

function TrovaPezzo(X,Y: integer; var p: TPiece; var pl: TPlayer): boolean;
var
  i: TPiece;
  j: TPlayer;
  val: TChbPosition;
begin
  val := CoordsToChbPosition(X,Y);
  result := false;

  for j := Low(TPlayer) to High(TPlayer) do
    for i := Low(Pieces[j]) to High(Pieces[j]) do
      if val = Pieces[j,i] then
      begin
        p := i; pl := j;
        result := true;
        break
      end
end;

procedure MuoviPezzo(Giocatore: TPlayer; Tipo: TPieceType; SrcX,SrcY,DstX,DstY: integer);
var
  p: TPiece;
  pl: TPlayer;
  DstVal: TChbPosition;
begin
  if TrovaPezzo(SrcX,SrcY,p,pl) and (Giocatore = pl) then
  begin
    dstval := CoordsToChbPosition(DstX,DstY);
    PieceTypes[pl,p] := Tipo;
    Pieces[pl,p] := dstVal;
  end
  else Beep
end;

procedure MangiaPezzo(X,Y: integer);
var
  p: TPiece;
  pl: TPlayer;
begin
  if TrovaPezzo(X,Y,p,pl) then
    Pieces[pl,p] := 0
  else Beep
end;

procedure FaiMossa(Pl: TPlayer; Mossa: string);
var
  Tipo: TPieceType;
  Giocatore: TPlayer;
  SrcX,SrcY: integer;
  DstX,DstY: integer;
  PresaX,PresaY: integer;
  Cnt: integer;
  function BaseRow(Pl: TPlayer): integer;
  begin
    if Pl = cplBlack then result := 8
    else result := 1
  end;
begin
  if Mossa = 'o-o' then
  begin
   MuoviPezzo(Pl,cptTower,8,BaseRow(Pl),6,BaseRow(Pl));
   MuoviPezzo(Pl,cptKing,5,BaseRow(Pl),7,BaseRow(Pl));
  end
  else if Mossa = 'o-o-o' then
  begin
   MuoviPezzo(Pl,cptTower,1,BaseRow(Pl),4,BaseRow(Pl));
   MuoviPezzo(Pl,cptKing,5,BaseRow(Pl),3,BaseRow(Pl));
  end
  else
  begin
    Cnt := 1;
    if Mossa[Cnt] = '!' then Inc(Cnt);
    getPieceTypeFromChar(Mossa[Cnt],Tipo,Giocatore);
    SrcX := StrToInt(Mossa[Cnt+1]);
    SrcY := StrToInt(Mossa[Cnt+2]);
    DstX := StrToInt(Mossa[Cnt+3]);
    DstY := StrToInt(Mossa[Cnt+4]);
    Inc(Cnt,5);
    if Length(Mossa) = (Cnt+2) then
    begin
      PresaX := StrToInt(Mossa[Cnt+1]);
      PresaY := StrToInt(Mossa[Cnt+2]);
      MangiaPezzo(PresaX,PresaY);
    end;
    MuoviPezzo(Pl,Tipo,SrcX,SrcY,DstX,DstY);
  end
end;


initialization
  RidisponiPezzi(false);
end.
