unit InternalEngine;

interface

uses SysUtils,Classes,Math,DebugU,AlfabetaU;

function Matto2Mosse(Topic: string; Board: string;
  DbgLevel: TDebugLevel = dlNone; StrList: TStrings = nil): string;
procedure CalcolaDettagli(Algorithm: TAlgorithm; Debug: TAbstractDebug;
  Depth: integer; Position: TAbstractPosition;
  var Moves1: TMovesList);

implementation

uses PositionU, MoveU, ParseStringU;

procedure CalcolaDettagli(Algorithm: TAlgorithm; Debug: TAbstractDebug;
  Depth: integer; Position: TAbstractPosition;
  var Moves1: TMovesList);
var
  CurrentMove: TAbstractMove;
  Cnt: integer;
  Value: integer;
  { mossa dummy }
  dummyMove: TAbstractMove;
begin
  Depth := Depth - 1;
  if (Depth >= 0) and Position.isLegal then
  begin
    Debug.Open;
    try
      Moves1 := nil;
      Cnt := 0;
      CurrentMove := Position.CreateMove;
      try
        CurrentMove.First(Position);
        if CurrentMove.isDefined then
          repeat
            Position.DoMove(CurrentMove);
            dummyMove := Position.CreateMove;
            try
              ChooseMove(Algorithm,Debug,Depth,Position,dummymove,Value);
              SetLength(Moves1,Cnt+2);
              Moves1[Cnt] := CurrentMove.AsString;
              Moves1[Cnt+1] := dummyMove.AsString;
              Inc(Cnt,2);
            finally
              dummyMove.Free;
            end;
            Position.UndoMove(CurrentMove);
          until not CurrentMove.Next(Position);
      finally
        CurrentMove.Free
      end;
    finally
      Debug.Close;
    end
  end
end;

function Matto2Mosse(Topic: string; Board: string;
  DbgLevel: TDebugLevel = dlNone; StrList: TStrings = nil): string;

  function AnswerToBool(answer: string): boolean;
  begin
    result := answer = 'y'
  end;
var
  Iniz,Finale: TDateTime;
  num: integer;
  params: TStringArray;
  Position: TPosition;
  Algo: TAlgorithm;
  Debug: TDebug;
  Valore: integer;
  CodErrore,Mossa,Tempo: string;
  RequireTime,RequireDetails: boolean;
  success: boolean;
  Mosse1: TMovesList;
  BestMove,dummyMove: TAbstractMove;
  i: integer;
begin
  { prima fase: lettura del topic }
  params := ParseString(Topic,',');
  // per un warning idiota devo inizializzarmi RT, RD e Algo
  RequireTime := false; RequireDetails := false; Algo := alMinimax;
  try
    num := Length(params);
    if num <> 3 then
      raise Exception.Create('topic malformato');
    if params[0] = 'minimax' then
      Algo := alMinimax
    else if params[0] = 'alfabeta' then
      Algo := alAlfabeta
    else
      raise Exception.Create('Algoritmo sconosciuto');
    RequireTime := AnswerToBool(params[1]);
    RequireDetails := AnswerToBool(params[2]);
  finally
    params := nil
  end;

  { seconda fase: chiamata dell'algoritmo }
  //  Debug := nil; // trucco cos posso usare un solo try
  Position := TPosition.Create;
  BestMove := Position.CreateMove;
  dummyMove := Position.CreateMove;
  if assigned(StrList) then
    Debug := TDebug.Create(DbgLevel,StrList)
  else
    Debug := TDebug.CreateFile(DbgLevel,'debug.txt');
  try
    try
      Position.SetPosition('bianco',Board);
    except
      result := 'scacchiera vuota';
      exit
    end;

    try
      if Position.isLeaf(Valore) then
      begin
        if Valore = 1 then CodErrore := 'scacco matto per il nero'
        else CodErrore := 'patta';
        Iniz := 0; Finale := 0;
      end
      else
      begin
        Iniz := Time;
        success := ChooseMove(Algo,Debug,3,Position,BestMove,Valore);
        Finale := Time;
        Mossa := BestMove.AsString;
        if success then
        begin
          if Valore = 1 then
          begin
            ChooseMove(Algo,Debug,1,Position,dummyMove,Valore);
            if Valore = 1 then
              CodErrore := 'il bianco matta in 1 mossa'
            else
            begin
              CodErrore := 'ok';
              if RequireDetails then
              begin
                Position.DoMove(BestMove);
                CalcolaDettagli(Algo,Debug,2,Position,Mosse1);
              end;
            end
          end
          else if Valore = -1 then CodErrore := 'il nero matta'
          else CodErrore := 'il bianco non matta in 2 mosse'
        end
        else CodErrore := 'posizione non legale (i re ci sono?)'
      end
    except
      Iniz := 0; Finale := 0;
      CodErrore := 'il bianco mangia il re nero';
    end
  finally
    Debug.Free;
    Position.Free;
    BestMove.Free;
    dummyMove.Free
  end;

  { terza fase: output }
  result := CodErrore;
  if CodErrore = 'il bianco matta in 1 mossa' then
    result := result + ',' + Mossa
  else if CodErrore = 'ok' then
  begin
    result := result + ',' + Mossa;
    if RequireTime then
    begin
      Tempo := IntToStr(Trunc((Finale-Iniz)*24*3600*1000));
      result := result + ',' + Tempo;
    end;
    if RequireDetails then
    begin
      for i := Low(Mosse1) to High(Mosse1) do
      begin
        result := result + ',' + Mosse1[i];
      end
    end
  end
end;

end.
