unit MoveU;

interface

uses SysUtils,Classes,AlfaBetaU,Utils;

type
  TMove = class(TAbstractMove)
  private
    FInternalCode: TMoveCode; // codifica la mossa del pezzo corrente
    FLegale: boolean;
    { Mosso }
    FPezzo: TTipoPezzo;
    FIdx: integer;  // indice nell'array
    FSrc: TCoords;
    FDst: TCoords;
    { Compagno }
    FCompagno: boolean;
//    FComPezzo: TTipoPezzo;  sempre il re
//    FCompagnoIdx: integer; sempre 0
    FCompagnoSrc: TCoords;
    FCompagnoDst: TCoords;
    { Trasformato }
    FTrasformato: boolean;
    FTrasfPezzo: TTipoPezzo;
    { Mangiato }
    FPresa: boolean;
    FPresaIdx: integer;
    FPresaPezzo: TTipoPezzo;
    FPresaCoords: TCoords;
    // utils
    function ProxMossa(Position: TAbstractPosition;
          IdxPezzo: integer; IdxMossa: TMoveCode): boolean;
    function ProxMossaSemiLegale(Position: TAbstractPosition;
          IdxPezzo: integer; IdxMossa: TMoveCode): boolean;
  public
    constructor Create;
    { movimento }
//    property Pezzo: TTipoPezzo read FPezzo;
    property IndicePezzo: integer read FIdx;
    property Source: TCoords read FSrc;
    property Dest: TCoords read FDst;
    { arrocco }
    property Arrocco: boolean read FCompagno;
//    property Compagno: TTipoPezzo read FComPezzo;
//    property ReIdx: integer read
    property ReSource: TCoords read FCompagnoSrc;
    property ReDest: TCoords read FCompagnoDst;
    { promozione }
    property Promozione: boolean read FTrasformato;
    property Promosso: TTipoPezzo read FTrasfPezzo;
    { presa }
    property Presa: boolean read FPresa;
    property PresaIdx: integer read FPresaIdx;
    property PezzoMangiato: TTipoPezzo read FPresaPezzo;
    property PresaDest: TCoords read FPresaCoords;
    { Interfaccia con l'algoritmo }
    { --------------------------- }
    // copia una mossa su un'altra
    procedure CopyFrom(SrcMove: TAbstractMove); override;
    // valore speciale che indica non definito
    // (usato nel caso ci sia gi una vittoria nella config. data)
    procedure setUndefined; override;
    // stabilisce se la mossa  definita
    // se non ci sono mosse legali deve impostare la mossa a undefined
    function IsDefined: boolean; override;
    // 2) move/2
    // riempie la move con la prima mossa legale valida nella posizione
    procedure First(Position: TAbstractPosition); override;
    // riempie la move con la prossima mossa legale nella posizione
    // se le mosse legali sono finite ritorna false e non modifica l'oggetto
    function Next(Position: TAbstractPosition): boolean; override;
    // 8) string_move
    // ritorna una rappresentazione della mossa in formato stringa
    // (per scopi di debug)
    function AsString: string; override;
  end;

implementation

uses PositionU, PezziU;

{ TMove }

constructor TMove.Create;
begin
  setUndefined { non sarebbe necessario comunque.. }
end;

// ***************************
// * IMPLEMENTAZIONE INTERNA *
// ***************************

{ genera SOLO mosse pienamente legali, cio verifica anche
  che il re non sia sotto assedio }
{ questo approccio si rende necessario perch l'algoritmo
  generico di minimax utilizzato non consente la generazione di
  mosse semi-legali }
function TMove.ProxMossa(Position: TAbstractPosition;
  IdxPezzo: integer; IdxMossa: TMoveCode): boolean;
var
//  DummyPosition: TPosition;
  done: boolean;
  Pos: TPosition;
begin
  Pos := TPosition(Position);
  done := false; result := false;
  FIdx := IdxPezzo;
  FInternalCode := IdxMossa;
  while not done do
  begin
    result := ProxMossaSemiLegale(Position,FIdx,FInternalCode);
    if result then
    begin
      { vai gi di un livello e controlla che non ci sia scacco al re }
      Pos.DoMove(self);
      if not Pos.ScaccoRe(Pos.Giocatore) then done := true;
      Pos.UndoMove(self);
    end
    else done := true;
  end
(*
  DummyPosition := TPosition.Create;
  try
    DummyPosition.CopyFrom(TPosition(Position));
    while not done do
    begin
      result := ProxMossaSemiLegale(Position,IdxPezzo,IdxMossa);
      if result then
      begin
        { vai gi di un livello e controlla che non ci sia scacco al re }
        DummyPosition.DoMove(self);
        if not DummyPosition.ScaccoRe(DummyPosition.Giocatore) then
          done := true
      end
      else done := true;
    end
  finally
    DummyPosition.Free
  end
*)
end;

{ cerca la prox mossa semilegale a partire da IdxPezzo, IdxMossa }
function TMove.ProxMossaSemiLegale(Position: TAbstractPosition;
  IdxPezzo: integer; IdxMossa: TMoveCode): boolean;
var
  Miei,Avversari: TList;
  Pos: TPosition;
  i, FoundIdx: integer;
  NewCode: integer;
  Tipo: TTipoPezzo;
  SrcCoords,DstCoords: TCoords;
  found: boolean;
  PezzoMangiato: integer;
begin
  Pos := TPosition(Position);
  Pos.getPezzi(Pos.Giocatore,Miei,Avversari);
  { cerca la prossima mossa }
  found := false;
  FoundIdx := -1;
  NewCode := IdxMossa;
  for i := IdxPezzo to Miei.Count-1 do
  begin
    PezzoMangiato := -1;
    Pos.EstraiPezzo(Miei,i,Tipo,SrcCoords);
    NewCode := NextCode(Tipo,Pos.Giocatore,SrcCoords,NewCode,true);
    while NewCode <> 0 do
    begin
      if DecodeMove(Pos.Giocatore,Tipo,SrcCoords,NewCode,DstCoords) and
         ( canMove(Position,NewCode,Tipo,SrcCoords,DstCoords) or
           canTake(Pos.Giocatore,Position,NewCode,Tipo,SrcCoords,
                   DstCoords,PezzoMangiato)) then
      begin
        found := true;
        break;
      end;
      NewCode := NextCode(Tipo,Pos.Giocatore,SrcCoords,NewCode,false)
    end;
    if found then
    begin
      FoundIdx := i;
      break;
    end
  end;
  { ora applicala }
  if found then
  begin
    FLegale := true;
//    ChangeMove(Pos.Giocatore,i,NewCode,Tipo,SrcCoords,DstCoords);
    FInternalCode := NewCode;
    FIdx := FoundIdx; // aggiorna FIdx
    { aggiorna FPezzo e FSrc }
    Pos.EstraiPezzo(Miei,FIdx,FPezzo,FSrc);
    FDst := DstCoords;
    if (FPezzo = t_torre) and ((NewCode = 1) or (NewCode = 2)) then
    begin
      FCompagno := true;
      FCompagnoSrc.X := 5;
      FCompagnoSrc.Y := RigaBase(Pos.Giocatore);
      if NewCode = 1 then FCompagnoDst.X := 7
      else FCompagnoDst.X := 3;
      FCompagnoDst.Y := RigaBase(Pos.Giocatore);
    end
    else FCompagno := false;
    if (FPezzo = t_pedone) and CanPromove(Pos.Giocatore,FSrc) then
    begin
      FTrasformato := true;
      if Hi(NewCode) = 0 then FTrasfPezzo := t_regina
      else FTrasfPezzo := t_cavallo
    end
    else FTrasformato := false;
    if PezzoMangiato <> -1 then
    begin
      FPresa := true;
      FPresaIdx := PezzoMangiato;
      Pos.EstraiPezzo(Avversari,FPresaIdx,FPresaPezzo,FPresaCoords)
    end
    else FPresa := false;
    result := true;
  end
  else result := false
end;

// **************************
// * INTERFACCIA COL MOTORE *
// **************************

function TMove.IsDefined: boolean;
begin
  result := FLegale
end;

procedure TMove.setUndefined;
begin
  FLegale := false
end;

procedure TMove.CopyFrom(SrcMove: TAbstractMove);
var
  SrcMv: TMove;
begin
  SrcMv := TMove(SrcMove);
(*
  System.Move(SrcMv, Self, SizeOf(SrcMv)); // non funziona
// ho provato a prenderlo da TMemortStream:
// Move(Buffer, Pointer(Longint(FMemory) + FPosition)^, Count);
*)
  FInternalCode := SrcMv.FInternalCode;
  FLegale := SrcMv.FLegale;
  { Mosso }
  FPezzo := SrcMv.FPezzo;
  FIdx := SrcMv.FIdx;
  FSrc := SrcMv.FSrc;
  FDst := SrcMv.FDst;
  { Compagno }
  FCompagno := SrcMv.FCompagno;
  FCompagnoSrc := SrcMv.FCompagnoSrc;
  FCompagnoDst := SrcMv.FCompagnoDst;
  { Trasformato }
  FTrasformato := SrcMv.FTrasformato;
  FTrasfPezzo := SrcMv.FTrasfPezzo;
  { Mangiato }
  FPresa := SrcMv.FPresa;
  FPresaIdx := SrcMv.FPresaIdx;
  FPresaPezzo := SrcMv.FPresaPezzo;
  FPresaCoords := SrcMv.FPresaCoords;
end;

function TMove.AsString: string;
begin
  if FLegale then
  begin
    if FCompagno and (FSrc.X = 1) then
      result := s_arrocco_lungo
    else if FCompagno and (FSrc.X = 8) then
      result := s_arrocco_corto
    else
    begin
      if FTrasformato then
        result := s_promozione + getCharFromPezzo(FTrasfPezzo,bianco)
      else
      result := getCharFromPezzo(FPezzo,bianco);
      result := result +
        IntToStr(FSrc.X) + IntToStr(FSrc.Y) + s_separatore +
        IntToStr(FDst.X) + IntToStr(FDst.Y);
      if FPresa then
        result := result + s_presa +
          IntToStr(FPresaCoords.X)  + IntToStr(FPresaCoords.Y);
    end
  end
  else result := s_mossa_illegale
end;

procedure TMove.First(Position: TAbstractPosition);
begin
  { c' sempre almeno un pezzo - il re - ma pu essere che non ci siano
    mosse - se  patta o scacco matto }
  if not ProxMossa(Position,0,0) then setUndefined;
end;

function TMove.Next(Position: TAbstractPosition): boolean;
begin
  { contrariamente alle specifiche anche se le mosse legali sono finite
    pu modificare l'oggetto (nel caso trovi mosse semi-legali).
    Comunque non dovrebbe dare problemi }
  result := ProxMossa(Position,FIdx,FInternalCode)
end;

end.
