unit PositionU;

interface

uses SysUtils,Classes,ParseStringU,Utils,AlfaBetaU;

type
  TPezzo = record
    Tipo: TTipoPezzo;
    // Colore: TColore; no, si desume dalla lista
    X,Y: integer
  end;
  TPosition = class(TAbstractPosition)
  private
    FGiocatore: TColore;
    FNeri: TList;
    FBianchi: TList;
    function setPlayer(pl: string): boolean;
    function setBoard(brd: string): boolean;
    procedure MuoviPezzo(Pezzi: TList; Idx: integer; Dest: TCoords);
    procedure TrasformaPezzo(Pezzi: TList; Idx: integer;
      Tipo: TTipoPezzo);
    procedure AggiungiPezzo(Pezzi: TList; Idx: integer; Tipo: TTipoPezzo;
                                      Dest: TCoords);
    procedure RimuoviPezzo(Pezzi: TList; Idx: integer);
    procedure LiberaMemoria;
  public
    property Giocatore: TColore read FGiocatore;

    constructor Create;
    destructor Destroy; override;
    { interfaccia verso l'esterno }
    { --------------------------- }
    // accesso pubblico alla scacchiera
//    property Chessboard: TChessboard read FChb write FChb;
    // imposta la posizione con delle stringhe
    procedure setPosition(pl: string; brd: string);
    { utils per TMove }
    { --------------- }
    procedure CopyFrom(Position: TPosition);
    procedure getPezzi(Giocatore: TColore; var Miei,Altri: TList);
    procedure EstraiPezzo(Pezzi: TList; idx: integer;
                         var Tipo: TTipoPezzo; var Coords: TCoords);
    function CasaOccupata(Coords: TCoords): boolean;
    function CercaPezzo(Pezzi: TList; Coords: TCoords): integer;
    function Scacco(Giocatore: TColore; Pezzi: TList; Coords: TCoords): boolean;
    function ScaccoRe(Giocatore: TColore): boolean;
    { Interfaccia con l'algoritmo }
    { --------------------------- }
    // istanzia uma TMove vuota
    function CreateMove: TAbstractMove; override;
    // 1) legal_position
    // stabilisce se la posizione  legale
    function isLegal: boolean; override;
    // 3) move/3
    // fa la mossa sulla schacchiera
    procedure DoMove(Mossa: TAbstractMove); override;
    // toglie la mossa dalla scacchiera
    procedure UndoMove(Mossa: TAbstractMove); override;
    // 4) is_leaf
    // parte della funzione di valutazione che controlla se  un nodo
    // terminale (vittoria, sconfitta o patta)
    function isLeaf(var Value: integer): boolean;
      override;
    // 5) euristics
    // tutto quello che resta fuori della funzione di valutazione
    // (Value viene ritornato)
    function Euristics: integer;
      override;
    // 6) won
    // controlla se Value  un valore di vittoria
    class function hasWon(Value: integer): boolean; override;
    // 7) string_position
    // ritorna una rappresentazione della posizione in formato stringa
    // (per scopi di debug)
    function AsString: string; override;
  end;

implementation

uses MoveU,PezziU;

{ TPosition }

constructor TPosition.Create;
begin
  FGiocatore := TColore(-1);
//  FReNeroIdx := -1;
//  FReBiancoIdx := -1;
  FNeri := TList.Create;
  FBianchi := TList.Create;
end;

destructor TPosition.Destroy;
begin
  LiberaMemoria;
end;

procedure TPosition.LiberaMemoria;
var
  i: integer;
begin
  for i := 0 to FNeri.Count-1 do Dispose(FNeri[i]);
  FNeri.Free;
  for i := 0 to FBianchi.Count-1 do Dispose(FBianchi[i]);
  FBianchi.Free;
end;

// *******************************
// * INTERFACCIA VERSO L'ESTERNO *
// *******************************

{ imposta la posizione con delle stringhe }
procedure TPosition.setPosition(pl: string; brd: string);
begin
  if not setPlayer(pl) or not setBoard(brd) then
    raise Exception.Create('cannot initialize position: wrong string');
end;

function TPosition.setBoard(brd: string): boolean;
var
  i: integer;
  params: TStringArray;
  Tipo: TTipoPezzo;
  Colore: TColore;
  X,Y: integer;
  PPezzo: ^TPezzo;
  Pezzi: TList;
begin
  result := false;
  params := ParseString(brd,',');
  try
    for i := Low(params) to High(params) do
      if (Length(params[i]) = 3) and
         getPezzoFromChar(params[i,1],Tipo,Colore) and
         parse_X(params[i,2],X) and
         parse_Y(params[i,3],Y) then
      begin
        new(PPezzo);
        PPezzo^.Tipo := Tipo;
        PPezzo^.X := X;
        PPezzo^.Y := Y;
        if Colore = bianco then Pezzi := FBianchi
        else Pezzi := FNeri;
        if Tipo = t_re then Pezzi.Insert(0,PPezzo)
        else Pezzi.Add(PPezzo);
        result := true; {ne basta uno corretto }
      end;
  finally
    params := nil
  end;
end;

function TPosition.setPlayer(pl: string): boolean;
begin
  if pl[1] = 'b' then
  begin
    FGiocatore := bianco;
    result := true
  end
  else if pl[1] = 'n' then
  begin
    FGiocatore := nero;
    result := true
  end
  else result := false
end;

(*
7) string_position
   ---------------
ritorna una rappresentazione della posizione in formato stringa
(per scopi di debug)
*)
function TPosition.AsString: string;
var
  i: integer;
  primo: boolean;
  Tipo: TTipoPezzo;
  Colore: TColore;
  Coords: TCoords;
  Pezzi: TList;
begin
  primo := true;
  result := '';
  for Colore := Low(TColore) to High(TColore) do
  begin
    if Colore = bianco then Pezzi := FBianchi
    else Pezzi := FNeri;
    for i := 0 to Pezzi.Count-1 do
    begin
      EstraiPezzo(Pezzi,i,Tipo,Coords);
      if primo then primo := false
      else result := result + ',';
      result := result + getCharFromPezzo(Tipo,Colore);
      result := result + code_X(Coords.X);
      result := result + code_Y(Coords.Y);
    end
  end
end;

// *****************
// * Utils private *
// *****************

procedure TPosition.MuoviPezzo(Pezzi: TList; Idx: integer; Dest: TCoords);
var
  PPezzo: ^TPezzo;
begin
  PPezzo := Pezzi[idx];
  PPezzo^.X := Dest.X;
  PPezzo^.Y := Dest.Y;
end;

procedure TPosition.TrasformaPezzo(Pezzi: TList; Idx: integer;
      Tipo: TTipoPezzo);
var
  PPezzo: ^TPezzo;
begin
  PPezzo := Pezzi[Idx];
  PPezzo^.Tipo := Tipo
end;

procedure TPosition.AggiungiPezzo(Pezzi: TList; Idx: integer; Tipo: TTipoPezzo;
                                  Dest: TCoords);
var
  PPezzo: ^TPezzo;
begin
  if (idx <> -1) then
  begin
    new(PPezzo);
    PPezzo^.Tipo := Tipo;
    PPezzo^.X := Dest.X;
    PPezzo^.Y := Dest.Y;
    Pezzi.Insert(Idx,PPezzo);
  end
end;

procedure TPosition.RimuoviPezzo(Pezzi: TList; Idx: integer);
var
  PPezzo: ^TPezzo;
begin
  if (idx <> -1) then
  begin
    PPezzo := Pezzi[idx];
    Pezzi.Delete(idx);
    Dispose(PPezzo)
  end
end;

// *******************
// * Utils pubbliche *
// *******************

procedure TPosition.getPezzi(Giocatore: TColore; var Miei, Altri: TList);
begin
  if Giocatore = bianco then
  begin
    Miei := FBianchi;
    Altri := FNeri
  end
  else
  begin
    Miei := FNeri;
    Altri := FBianchi
  end;
end;

procedure TPosition.EstraiPezzo(Pezzi: TList; idx: integer;
  var Tipo: TTipoPezzo; var Coords: TCoords);
var
  PPezzo: ^TPezzo;
begin
  PPezzo := Pezzi[idx];
  Tipo := PPezzo^.Tipo;
  Coords.X := PPezzo^.X;
  Coords.Y := PPezzo^.Y;
end;

function TPosition.CasaOccupata(Coords: TCoords): boolean;
var
  Miei,Altri: TList;
begin
  getPezzi(FGiocatore,Miei,Altri);
  if (CercaPezzo(Miei,Coords) = -1) and (CercaPezzo(Altri,Coords) = -1) then
    result := false
  else result := true
end;

function TPosition.CercaPezzo(Pezzi: TList; Coords: TCoords): integer;
var
  i: integer;
  Tipo: TTipoPezzo;
  PieceCoords: TCoords;
begin
  result := -1;
  for i := 0 to Pezzi.Count-1 do
  begin
    EstraiPezzo(Pezzi,i,Tipo,PieceCoords);
    if (Coords.X = PieceCoords.X) and (Coords.Y = PieceCoords.Y) then
    begin
      result := i;
      break
    end
  end
end;

function TPosition.ScaccoRe(Giocatore: TColore): boolean;
var
  Miei,Altri: TList;
  ReAvvTipo: TTipoPezzo;
  ReAvvCoords: TCoords;
begin
  getPezzi(Giocatore,Miei,Altri);
  EstraiPezzo(Altri,0,ReAvvTipo,ReAvvCoords);
  result := Scacco(Giocatore,Miei,ReAvvCoords);
end;

function TPosition.Scacco(Giocatore: TColore; Pezzi: TList;
                          Coords: TCoords): boolean;
var
  i: integer;
  Tipo: TTipoPezzo;
  SrcCoords: TCoords;
begin
  result := false;
  for i := 0 to Pezzi.Count-1 do
  begin
    EstraiPezzo(Pezzi,i,Tipo,SrcCoords);
    if minaccia(Giocatore,Self,Tipo,SrcCoords,Coords) then
    begin
      result := true;
      break
    end
  end
end;

// **************************
// * INTERFACCIA COL MOTORE *
// **************************

function TPosition.CreateMove: TAbstractMove;
begin
  result := TMove.Create
end;

(*
1) legal_position
   ---------------
stabilisce se la posizione  legale
CONTROLLI:
1 - Giocatore = bianco|nero?
2 - Neri e Bianchi sono 2 vettori di max 16 elem. di triple?
3 - ogni tripla  costituita da Tipo,X,Y corretti?
    (Tipo=torre,cavallo,alfiere,regina,re,pedone,1<=X,Y<=8)
4 - i re esistono e sono in testa? non ci sono altri re?
5 - esistono coppie di coordinate duplicate?
CONTROLLI NON EFFETTUATI:
- se ci sono pi pezzi di un tipo di quelli possibili
  (l'unico controllo  sui re) Questo permette di
  testare il motore con situazioni poco verosimili
*)
function TPosition.isLegal: boolean;
  function corretto(Pezzi: TList): boolean;
  { verifica:
    1 - che la lunghezza della lista sia al max 16
    2 - che ogni singolo pezzo sia ben formato
    3 -che esista uno e un solo re e che sia in testa }
  var
    i: integer;
    TipoRe,Tipo: TTipoPezzo;
    Coords: TCoords;
  begin
    if Pezzi.Count > 0 then
    begin
      EstraiPezzo(Pezzi,0,TipoRe,Coords);
      if (TipoRe = t_re) and (Pezzi.Count <= 16) then
      begin
        result := true;
        for i := 0 to Pezzi.Count-1 do
        begin
          EstraiPezzo(Pezzi,i,Tipo,Coords);
          if (Tipo < Low(TTipoPezzo)) or (Tipo > High(TTipoPezzo)) or
             (Coords.X < 1) or (Coords.X > 8) or
             (Coords.Y < 1) or (Coords.Y > 8) or
             ((Tipo = t_re) and (i <> 0)) then
          begin
            result := false;
            break
          end
        end
      end
      else result := false
    end
    else result := false
  end;
  { verifica che non ci siano pi pezzi nelle stesse caselle }
  function verifica_coordinate(Neri,Bianchi: TList): boolean;
  var
    occupato: array[1..8,1..8] of boolean;
    i,j: integer;
    Tipo: TTipoPezzo;
    Coords: TCoords;
  begin
    result := true;
    for i := 1 to 8 do
      for j := 1 to 8 do occupato[i,j] := false;
    for i := 0 to Neri.Count-1 do
    begin
      EstraiPezzo(Neri,i,Tipo,Coords);
      if occupato[Coords.Y,Coords.X] then
      begin
        result := false;
        break
      end
      else occupato[Coords.Y,Coords.X] := true
    end
  end;
begin
  if ((FGiocatore = bianco) or (FGiocatore = nero)) and
     corretto(FNeri) and corretto(FBianchi) and
     verifica_coordinate(FNeri,FBianchi) then
    result := true
  else result := false
end;

(*
3) move/3
   ------
fa la mossa sulla schacchiera
esegue una mossa (controllo di legalit gi eseguito)
(data la mossa aggiorna la posizione)
*)

procedure TPosition.DoMove(Mossa: TAbstractMove);
var
  Miei,Avversari: TList;
begin
  getPezzi(FGiocatore,Miei,Avversari);
  { Fai la mossa }
  with TMove(Mossa) do
  begin
    MuoviPezzo(Miei,IndicePezzo,Dest);
    { mosse speciali }
    if Arrocco then MuoviPezzo(Miei,0,ReDest)
    else if Promozione then TrasformaPezzo(Miei,IndicePezzo,Promosso);
    { presa }
    if Presa then RimuoviPezzo(Avversari,PresaIdx);
  end;
  { Cambia giocatore }
  if FGiocatore = bianco then FGiocatore := nero
  else FGiocatore := bianco
end;

procedure TPosition.UndoMove(Mossa: TAbstractMove);
var
  Miei,Avversari: TList;
begin
  { Cambia giocatore }
  if FGiocatore = bianco then FGiocatore := nero
  else FGiocatore := bianco;
  { da farsi dopo cambia giocatore }
  getPezzi(FGiocatore,Miei,Avversari);
  { Disfa la mossa }
  with TMove(Mossa) do
  begin
    { presa }
    { prima ripristina il pezzo cos gli indici tornano ad essere corretti }
    if Presa then AggiungiPezzo(Avversari,PresaIdx,PezzoMangiato,PresaDest);
    { ora fai tornare indietro le mosse }
    MuoviPezzo(Miei,IndicePezzo,Source);
    { mosse speciali }
    if Arrocco then MuoviPezzo(Miei,0,ReSource)
    else if Promozione then TrasformaPezzo(Miei,IndicePezzo,t_pedone);
  end;
end;

(*
4) is_leaf
   -------
parte della funzione di valutazione che controlla se  un nodo
terminale (vittoria, sconfitta o patta)
*)
function TPosition.isLeaf(var Value: integer): boolean;
var
  dummyMove: TMove;
  defined: boolean;
  Avversario: TColore;
begin
  { nessuna mossa? allora  patta o scacco matto }
  dummyMove := TMove(CreateMove);
  try
    dummyMove.First(self);
    defined := dummyMove.IsDefined;
  finally
    dummyMove.Free
  end;
  if not defined then
  begin
    { se c' anche lo scacco allora  matto }
    if FGiocatore = bianco then Avversario := nero
    else Avversario := bianco;
    if ScaccoRe(Avversario) then
      value := 1
    else value := 0;
    result := true
  end
  else result := false;
end;

(*
5) euristics
   ---------
tutto quello che resta fuori della funzione di valutazione
 (Value viene ritornato)
*)
function TPosition.Euristics: integer;
begin
  // nessuna euristica
  result := 0
end;

(*
6) won
   ---
controlla se Value  un valore di vittoria
NOTA: Viene usato solo da minimax (NON da alfabeta)
NB2: in futuro anche alfabeta potrebbe usarlo
*)
class function TPosition.hasWon(Value: integer): boolean;
begin
  result := Value = 1
end;

procedure TPosition.CopyFrom(Position: TPosition);
  procedure CopiaPezzi(Src: TList; Dst: TList);
  var
    Tipo: TTipoPezzo;
    Coords: TCoords;
    i: integer;
  begin
    for i := 0 to Src.Count-1 do
    begin
      EstraiPezzo(Src,i,Tipo,Coords);
      AggiungiPezzo(Dst,i,Tipo,Coords);
    end
  end;
begin
  FGiocatore := Position.FGiocatore;
  LiberaMemoria;
  CopiaPezzi(Position.FNeri,FNeri);
  CopiaPezzi(Position.FBianchi,FBianchi);
end;

end.
