unit Setup;

interface

{$WARN UNIT_PLATFORM OFF } // serve per FileCtrl

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, FileCtrl, StdCtrls, Buttons, ExtCtrls,Registry, IniFiles,
  Game, ComCtrls, DdeMan;

type
  TLinkType = (ltInternal,ltDDE);
  TAlgorithm = (alMinimax,alAlfabeta);

  TGfxSet = record
    Loaded: boolean; // contiene valori coerenti?
    { header }
    Description: string; // forse non dovrebbe stare qui ma dove allora?
    { configurazione salvata sui pezzi }
    PiecesFilename: string;
    BackCol: TColor;
    PiecesLeftB,PiecesTopB: integer;
    PiecesW,PiecesH: integer;
    PiecesHSep,PiecesVSep: integer;
    { configurazione salvata sulla scacchiera }
    ChessboardFilename: string;
    SquareW,SquareH: integer;
    LeftBorder,TopBorder: integer;
    HSeparation,VSeparation: integer;
    { configurazione su textures e cursori }
    TexturesFilename: string;
    CursorOpenHand: string;
    CursorCloseHand: string;
  end;

  TSetupFrm = class(TForm)
    PageControl1: TPageControl;
    GfxSheet: TTabSheet;
    EngineSheet: TTabSheet;
    FileListBox1: TFileListBox;
    ButtonPnl: TPanel;
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    OpenDialog1: TOpenDialog;
    OpzioniBox: TGroupBox;
    MinimaxRadio: TRadioButton;
    AlfaBetaRadio: TRadioButton;
    TimeChk: TCheckBox;
    DetailsChk: TCheckBox;
    DdeClientConv1: TDdeClientConv;
    LinkBox: TGroupBox;
    InternalEngineRadio: TRadioButton;
    DDELinkRadio: TRadioButton;
    DDEPnl: TPanel;
    ServiceBtn: TSpeedButton;
    ServiceLbl: TLabel;
    ConnectBtn: TButton;
    ServiceCombo: TComboBox;
    MainPnl: TPanel;
    PreviewBox: TGroupBox;
    PreviewImg: TImage;
    TopPnl: TPanel;
    DescriptionLbl: TLabel;
    DescriptionEdt: TEdit;
    procedure FormShow(Sender: TObject);
    procedure FileListBox1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FileListBox1DblClick(Sender: TObject);
    procedure ConnectBtnClick(Sender: TObject);
    procedure InternalEngineRadioClick(Sender: TObject);
    procedure DDELinkRadioClick(Sender: TObject);
    procedure ServiceBtnClick(Sender: TObject);
    procedure OkBtnClick(Sender: TObject);
  private
    { Private declarations }
    PreviewTextureBmp,PreviewChessboardBmp: TBitmap;
    PreviewPiecesBmp: array[TPlayer,TPieceType] of TBitmap;
    ChbLeft,ChbTop: integer;
    ChbWidth,ChbHeight: integer;
    procedure CaricaAnteprima;
    procedure VisualizzaAnteprima;
    procedure CaricaDaFile(Filename: string);
    procedure SaveGfxToReg(reg: TRegistry);
    procedure DisegnaPezzoAnteprima(campo: TPlayer; n: TPiece);
    procedure AggiornaCombo(combo: TComboBox);
    procedure LoadGfxFromReg(reg: TRegistry);
  public
    { Public declarations }
    LinkType: TLinkType;
    DDEService: string;
    Algorithm: TAlgorithm;
    RequireTime: boolean;
    RequireDetails: boolean;
    
    PreviewGfxSet: TGfxSet;
    function CaricaDaRegistro: boolean;
    procedure SalvaSuRegistro;
    function DDETopic: string;
    function Connetti(flag: boolean): boolean;
  end;

const
  GfxDir = 'Graphics sets';

var
  SetupFrm: TSetupFrm;

implementation

uses ModuloDati;

{$R *.dfm}

// GRAPHICS
// --------

procedure TSetupFrm.LoadGfxFromReg(reg: TRegistry);
begin
  with PreviewGfxSet do
  begin
    Reg.CloseKey;
    if Reg.OpenKeyReadOnly(ChiaveRegistro+'\Gfx set') then
    begin
      { header }
      Description := Reg.ReadString('Description');
      { pieces storage }
      PiecesFilename := Reg.ReadString('Pieces file');
      BackCol := Reg.ReadInteger('Pieces transparent color');
      PiecesLeftB := Reg.ReadInteger('Pieces left border');
      PiecesTopB := Reg.ReadInteger('Pieces right border');
      PiecesW := Reg.ReadInteger('Pieces width');
      PiecesH := Reg.ReadInteger('Pieces height');
      PiecesHSep := Reg.ReadInteger('Pieces horizontal separation');
      PiecesVSep := Reg.ReadInteger('Pieces vertical separation');
      { chessboard storage }
      ChessboardFilename := Reg.ReadString('Chessboard file');
      LeftBorder := Reg.ReadInteger('Chessboard left border');
      TopBorder := Reg.ReadInteger('Chessboard right border');
      SquareW := Reg.ReadInteger('Squares width');
      SquareH := Reg.ReadInteger('Squares height');
//      HSeparation := Reg.ReadInteger('Squares horizontal separation');
//      VSeparation := Reg.ReadInteger('Squares vertical separation');
      { other }
      TexturesFilename := Reg.ReadString('Texture file');
      CursorOpenHand := Reg.ReadString('Cursor file 1');
      CursorCloseHand := Reg.ReadString('Cursor file 2');
      Reg.CloseKey;
      Loaded := true
    end;
  end
end;

procedure TSetupFrm.SaveGfxToReg(reg: TRegistry);
begin
  if PreviewGfxSet.Loaded then
    with PreviewGfxSet do
    begin
      Reg.CloseKey;
      if Reg.OpenKey(ChiaveRegistro+'\Gfx set', True) then
      begin
        { header }
        Reg.WriteString('Description',Description);
        { pieces storage }
        Reg.WriteString('Pieces file',PiecesFilename);
        Reg.WriteInteger('Pieces transparent color',BackCol);
        Reg.WriteInteger('Pieces left border',PiecesLeftB);
        Reg.WriteInteger('Pieces right border',PiecesTopB);
        Reg.WriteInteger('Pieces width',PiecesW);
        Reg.WriteInteger('Pieces height',PiecesH);
        Reg.WriteInteger('Pieces horizontal separation',PiecesHSep);
        Reg.WriteInteger('Pieces vertical separation',PiecesVSep);
        { chessboard storage }
        Reg.WriteString('Chessboard file',ChessboardFilename);
        Reg.WriteInteger('Chessboard left border',LeftBorder);
        Reg.WriteInteger('Chessboard right border',TopBorder);
        Reg.WriteInteger('Squares width',SquareW);
        Reg.WriteInteger('Squares height',SquareH);
//        Reg.WriteInteger('Squares horizontal separation',HSeparation);
//        Reg.WriteInteger('Squares vertical separation',VSeparation);
        { other }
        Reg.WriteString('Texture file',TexturesFilename);
        Reg.WriteString('Cursor file 1',CursorOpenHand);
        Reg.WriteString('Cursor file 2',CursorCloseHand);
        Reg.CloseKey;
      end;
    end
end;

procedure TSetupFrm.CaricaAnteprima;
var
  AllPiecesBmp: TBitmap;
  SrcRect,DstRect: TRect;
  pl: TPlayer;
  n: TPieceType;
  FullGfxDir: string;
begin
  if PreviewGfxSet.Loaded then
    with PreviewGfxSet do
    begin
      FullGfxDir := ExtractFilePath(Application.ExeName) + GfxDir + '\';
      { Descrizione }
      DescriptionEdt.Text := Description;
      { Pezzi }
      if PiecesFilename <> '' then
      begin
        AllPiecesBmp := TBitmap.Create;
        try
          AllPiecesBmp.LoadFromFile(FullGfxDir+PiecesFilename);
          DstRect := Rect(0,0,PiecesW,PiecesH);
          for pl := Low(TPlayer) to High(TPlayer) do
            for n := Low(PreviewPiecesBmp[pl]) to High(PreviewPiecesBmp[pl]) do
            begin
              PreviewPiecesBmp[pl,n].Width := PiecesW;
              PreviewPiecesBmp[pl,n].Height := PiecesH;
              PreviewPiecesBmp[pl,n].Transparent := True;
              PreviewPiecesBmp[pl,n].TransparentColor := BackCol;

              SrcRect := Rect(PiecesLeftB+(PiecesW+PiecesHSep)*ord(n),
                              PiecesTopB,
                              PiecesLeftB+(PiecesW+PiecesHSep)*(ord(n)+1)-PiecesHSep,
                              PiecesTopB+PiecesH);
              if pl = cplWhite then
              begin
                Inc(SrcRect.Top,PiecesH+PiecesVSep);
                Inc(SrcRect.Bottom,PiecesH+PiecesVSep);
              end;

              PreviewPiecesBmp[pl,n].Canvas.CopyRect(DstRect,AllPiecesBmp.Canvas,SrcRect);
            end
        finally
          AllPiecesBmp.Free;
        end;
      end;

      { Chessboard }
      if ChessboardFilename <> '' then
        PreviewChessboardBmp.LoadFromFile(FullGfxDir+ChessboardFilename);

      { Textures }
      if TexturesFilename <> '' then
        PreviewTextureBmp.LoadFromFile(FullGfxDir+TexturesFilename);
    end
end;

procedure TSetupFrm.CaricaDaFile(Filename: string);
var
  FullPathName: string;
  BackColString: string;
  IniFile: TIniFile;
begin
  FullPathName := {ExtractFilePath(Application.ExeName) + GfxDir +} Filename;
  if not FileExists(FullPathName) then
    raise Exception.Create('File di configurazione non trovato');

  IniFile := TIniFile.Create(FullPathName);
  try
    with PreviewGfxSet do
    begin
      Loaded := false; // iniziamo con porre a false il flag di coerenza

      if IniFile.SectionExists('Header') then
      begin
        Description := IniFile.ReadString('Header','Description','')
      end;
      // else pazienza

      if IniFile.SectionExists('Pieces') then
      begin
        PiecesFilename := IniFile.ReadString('Pieces','Filename','');
        if PiecesFilename = '' then
          raise Exception.Create('File dei pezzi non specificato');
        BackColString := IniFile.ReadString('Pieces','BackGroundColor','');
        if BackColString <> '' then BackCol := StringToColor(BackColString)
        else
          raise Exception.Create('Colore di background non trovato o errato');
        PiecesLeftB := IniFile.ReadInteger('Pieces','LeftBorder',-1);
        PiecesTopB := IniFile.ReadInteger('Pieces','TopBorder',-1);
        if (PiecesLeftB = -1) or (PiecesTopB = -1) then
          raise Exception.Create('Bordi dei pezzi non specificati o fuori range');
        PiecesW := IniFile.ReadInteger('Pieces','PieceWidth',0);
        PiecesH := IniFile.ReadInteger('Pieces','PieceHeight',0);
        if (PiecesW = 0) or (PiecesH = 0) then
          raise Exception.Create('Dimensioni dei pezzi non specificate o fuori range');
        PiecesHSep := IniFile.ReadInteger('Pieces','HSeparation',-1);
        PiecesVSep := IniFile.ReadInteger('Pieces','VSeparation',-1);
        if (PiecesHSep = -1) or (PiecesVSep = -1) then
          raise Exception.Create('Separatori tra i pezzi non specificati o fuori range');
      end
      else
        raise Exception.Create('Dov'' la sezione dedicata ai pezzi?');

      if IniFile.SectionExists('Chessboard') then
      begin
        ChessboardFilename := IniFile.ReadString('Chessboard','Filename','');
        if ChessboardFilename = '' then
          raise Exception.Create('Scacchiera non specificata');
        SquareW := IniFile.ReadInteger('ChessBoard','SquareWidth',0);
        SquareH := IniFile.ReadInteger('ChessBoard','SquareHeight',0);
        if (SquareW = 0) or (SquareH = 0) then
          raise Exception.Create('Dimensioni delle caselle non trovate o fuori range');
        LeftBorder := IniFile.ReadInteger('ChessBoard','LeftBorder',-1);
        TopBorder := IniFile.ReadInteger('ChessBoard','TopBorder',-1);
        if (LeftBorder = -1) or (TopBorder = -1) then
          raise Exception.Create('Bordi della scacchiera non specificati o fuori range');
        HSeparation := IniFile.ReadInteger('ChessBoard','HSeparation',-1);
        VSeparation := IniFile.ReadInteger('ChessBoard','VSeparation',-1);
        if (HSeparation = -1) or (VSeparation = -1) then
          raise Exception.Create('Separatori tra le caselle non specificati o fuori range');
      end
      else raise Exception.Create('Dov'' la sezione dedicata alla scacchiera?');

      if IniFile.SectionExists('Textures') then
      begin
        TexturesFilename := iniFile.ReadString('Textures','Filename','');
        // else pazienza
      end;
      // else - se le textures non ci sono pazienza

      if IniFile.SectionExists('Cursors') then
      begin

      end;
      // else - se non ci sono i cursori pazienza

      Loaded := true; // tutto ok, flag di coerenza su
    end
  finally
    IniFile.Free
  end
end;

procedure TSetupFrm.DisegnaPezzoAnteprima(campo: TPlayer; n: TPiece);
var
  DstLeft,DstTop: integer;
  BmpIdx: TPieceType;
  Idx: integer;
begin
  BmpIdx := PieceTypes[campo,n];
  Idx := ord(n);
  if campo = cplWhite then Inc(Idx,48);
  { indirizzo 0,0 - in realt vanno centrate }
  with PreviewGfxSet do
  begin
    DstLeft := ChbLeft + LeftBorder + (Idx mod NumSquaresW) * SquareW;
    DstTop := ChbTop + TopBorder + (Idx div NumSquaresH) * SquareH;
  end;
  PreviewImg.Canvas.Draw(DstLeft,DstTop,PreviewPiecesBmp[campo,BmpIdx]);
end;

procedure TSetupFrm.VisualizzaAnteprima;
var
  pl: TPlayer;
  i: TPiece;
  SrcRect,DstRect: TRect;
begin
  if PreviewGfxSet.Loaded then
  begin
    { disegna le textures }
    if assigned(PreviewTextureBmp) then
    begin
      with PreviewImg do
      begin
        Canvas.Brush.Bitmap := PreviewTextureBmp;
        Canvas.FillRect(Rect(0,0,Width,Height));
        PreviewImg.Canvas.Brush.Bitmap := nil;
      end;
    end;

    { disegna la scacchiera }
    if assigned(PreviewChessboardBmp) then
    begin
      ChbWidth := PreviewChessboardBmp.Width;
      ChbHeight := PreviewChessboardBmp.Height;
      ChbLeft := (PreviewImg.Width - ChbWidth) div 2;
      ChbTop := (PreviewImg.Height - ChbHeight) div 2;
      SrcRect := Rect(0,0,ChbWidth,ChbHeight);
      DstRect := Rect(ChbLeft,ChbTop,ChbLeft+ChbWidth,ChbTop+ChbHeight);
      PreviewImg.Canvas.CopyRect(DstRect,PreviewChessboardBmp.Canvas,SrcRect);
    end;

    { disegna i pezzi }
    if assigned(PreviewPiecesBmp[cplBlack,Low(PreviewPiecesBmp[cplBlack])]) then
    begin
      for pl := Low(TPlayer) to High(TPlayer) do
        for i := Low(Pieces[pl]) to High(Pieces[pl]) do
          DisegnaPezzoAnteprima(pl,i);
    end
  end
end;

// REGISTRO
// --------

function TSetupFrm.CaricaDaRegistro: boolean;
var
  Reg: TRegistry;
  InternalEngine: boolean;
begin
  reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKeyReadOnly(ChiaveRegistro) then
    begin
      InternalEngine := Reg.ReadBool('Internal engine');
      if InternalEngine then InternalEngineRadio.Checked := true
      else DDELinkRadio.Checked := true;
      ServiceCombo.Text := Reg.ReadString('DDE Service');
      ServiceCombo.Items.CommaText := Reg.ReadString('DDE Server list');
//      if assigned(DDESetupForm) then DDESetupForm.LoadFromReg(Reg)
//      else ShowMessage('Errore: non  ancora caricato');
      Reg.CloseKey;
      LoadGfxFromReg(Reg); {lo lascio per ultimo perch chiude la chiave }
      result := true
    end
    else result := false
  finally
    Reg.Free;
  end;
end;

procedure TSetupFrm.SalvaSuRegistro;
var
  Reg: TRegistry;
begin
  reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(ChiaveRegistro, True) then
    begin
      Reg.WriteBool('Internal engine',InternalEngineRadio.Checked);
      Reg.WriteString('DDE Service',ServiceCombo.Text);
      Reg.WriteString('DDE Server list',ServiceCombo.Items.CommaText);
//      if assigned(DDESetupForm) then DDESetupForm.SaveToReg(Reg);
      Reg.CloseKey;
    end;
    SaveGfxToReg(Reg); {lo lascio per ultimo perch chiude la chiave }
  finally
    Reg.Free;
  end;
end;

// LINK DDE
// --------

function TSetupFrm.DDETopic: string;
begin
  case Algorithm of
  alMinimax: result := 'minimax';
  alAlfabeta: result := 'alfabeta';
  end;
  if requireTime then
    result := result + ',y'
  else result := result + ',n';
  if requireDetails then
    result := result + ',y'
  else result := result + ',n';
end;

{
Connessione al server DDE
 flag = true se richiamato dall'interno
 flag = false se  richiamato dall'esterno
}
function TSetupFrm.Connetti(flag: boolean): boolean;
begin
  if (flag and (LinkType = ltDDE)) or
     (not flag and DDELinkRadio.Checked) then
    result :=
      DDEClientConv1.SetLink(ServiceCombo.Text,DDETopic) or
      { secondo tentativo }
      DDEClientConv1.SetLink(ServiceCombo.Text,DDETopic) 
  else result := true;
end;

// INTERFACCIA GRAFICA
// -------------------

procedure TSetupFrm.FormCreate(Sender: TObject);
var
  pl: TPlayer;
  n: TPieceType;
begin
  DDEService := 'prologEngine';
  Algorithm := alAlfabeta;
  RequireTime := true;
  RequireDetails := true;
  LinkType := ltInternal;
  InternalEngineRadioClick(Sender);

  for pl := Low(TPlayer) to High(TPlayer) do
    for n := Low(PreviewPiecesBmp[pl]) to High(PreviewPiecesBmp[pl]) do
      PreviewPiecesBmp[pl,n] := TBitmap.Create;
  PreviewChessboardBmp := TBitMap.Create;
  PreviewTextureBmp := TBitmap.Create;
end;

procedure TSetupFrm.FormDestroy(Sender: TObject);
var
  i: TPieceType;
  pl: TPlayer;
begin
  for pl := Low(TPlayer) to High(TPlayer) do
    for i := Low(PreviewPiecesBmp[pl]) to High(PreviewPiecesBmp[pl]) do
      if assigned(PreviewPiecesBmp[pl,i]) then PreviewPiecesBmp[pl,i].Free;
  if assigned(PreviewChessboardBmp) then PreviewChessboardBmp.Free;
  if assigned(PreviewTextureBmp) then PreviewTextureBmp.Free;
end;

procedure TSetupFrm.FormResize(Sender: TObject);
begin
  if PreviewGfxSet.Loaded then
    with PreviewImg do
    begin
      Picture.Bitmap.Width := Width;
      Picture.Bitmap.Height := Height;
      VisualizzaAnteprima
    end;
end;

procedure TSetupFrm.FormShow(Sender: TObject);
begin
  case LinkType of
  ltInternal: InternalEngineRadio.Checked := true;
  ltDDE: DDELinkRadio.Checked := true
  end;
  ServiceCombo.Text := DDEService;
  case Algorithm of
  alMinimax: MinimaxRadio.Checked := true;
  alAlfabeta: AlfabetaRadio.Checked := true
  end;
  TimeChk.Checked := RequireTime;
  DetailsChk.Checked := RequireDetails;

  FileListBox1.Drive := ExtractFileDrive(Application.ExeName)[1];
  FileListBox1.Directory := ExtractFilePath(Application.ExeName) + GfxDir;
  if PreviewGfxSet.Loaded then
  begin
    CaricaAnteprima;
    VisualizzaAnteprima;
  end
end;

procedure TSetupFrm.FileListBox1Change(Sender: TObject);
begin
  if FileListBox1.ItemIndex >= 0 then
  begin
    CaricaDaFile(FileListBox1.FileName);
    CaricaAnteprima;
    VisualizzaAnteprima
  end
end;

procedure TSetupFrm.FileListBox1DblClick(Sender: TObject);
begin
  ModalResult := mrOk;
end;

procedure TSetupFrm.ConnectBtnClick(Sender: TObject);
begin
  if (ServiceCombo.Text <> '') and Connetti(false) then
  begin
    //DDEClientItem1.DdeItem := ...;
    ShowMessage('Connessione stabilita con successo');
    AggiornaCombo(ServiceCombo);
    DDEClientConv1.CloseLink;
  end
  else ShowMessage('Connessione fallita')
end;

procedure TSetupFrm.InternalEngineRadioClick(Sender: TObject);
begin
// LinkBox.Height := 60;
  DDEPnl.Visible := false
end;

procedure TSetupFrm.DDELinkRadioClick(Sender: TObject);
begin
//  LinkBox.Height := 134;
  DDEPnl.Visible := true
end;

procedure TSetupFrm.ServiceBtnClick(Sender: TObject);
var
  ServiceDir,ServiceName: string;
  idx: integer;
begin
  if ServiceCombo.Text <> '' then
    OpenDialog1.FileName := ServiceCombo.Text;

  if OpenDialog1.Execute then
  begin
    ServiceName := ExtractFileName(OpenDialog1.FileName);
    { tolgo l'estensione }
    idx := LastDelimiter('.',ServiceName);
    if idx>0 then ServiceName := Copy(ServiceName,1,idx-1);
    { ora aggiungo eventualmente il percorso }
    ServiceDir := ExtractFilePath(OpenDialog1.FileName);
    if ExtractFilePath(Application.ExeName) <> ServiceDir then
      ServiceName := ServiceDir + ServiceName;
    { aggiorno il combo }
    ServiceCombo.Text := ServiceName;
    AggiornaCombo(ServiceCombo);
  end
end;

procedure TSetupFrm.AggiornaCombo(combo: TComboBox);
var
  idx: integer;
  count: integer;
begin
  idx := combo.Items.IndexOf(combo.text);
  if idx = -1 then
    combo.Items.Insert(0,combo.text)
  else
    combo.Items.Move(idx,0);

  count := combo.DropDownCount;
  while (count > 0) and (combo.Items.Count > count) do
    combo.Items.Delete(count);
end;

procedure TSetupFrm.OkBtnClick(Sender: TObject);
begin
  if InternalEngineRadio.Checked then
    LinkType := ltInternal
  else LinkType := ltDDE;
  DDEService := ServiceCombo.Text;
  if MinimaxRadio.Checked then
    Algorithm := alMinimax
  else Algorithm := alAlfabeta;
  RequireTime := TimeChk.Checked;
  RequireDetails := DetailsChk.Checked
end;

end.
