% gioca filetto
% programma che gioca a filetto 
% Permette di scegliere tra minimax puro o con alfabeta

% carico le regole del gioco
user:file_search_path(engine,'../engine').
:- [regole].

gioca :- gioca(alfabeta,9).

% gioca(+Algoritmo,+Depth).
% chiede i giocatori e richiama l'algoritmo giusto
gioca(Algoritmo,MaxDepth) :-
  carica_algoritmo(Algoritmo),
  scrivi_presentazione(Algoritmo,MaxDepth),
  chiedi_giocatore(1,PlType1),
  chiedi_giocatore(2,PlType2),nl,
%  Scacchiera="         ",
  Scacchiera='         ',
  PlayerInfo=(1,PlType1,PlType2),
  mostra_scacchiera(Scacchiera),
  play(Algoritmo,MaxDepth,Scacchiera,PlayerInfo,_).

% i due algoritmi supportati: minimax puro o con potatura alfabeta
carica_algoritmo(minimax) :- ensure_loaded(engine(minimax)).
carica_algoritmo(minimax_simple) :- ensure_loaded(engine(minimax)).
carica_algoritmo(alfabeta) :- ensure_loaded(engine(alfabeta)).

scrivi_presentazione(Algoritmo,MaxDepth) :-
  writeln('Gioco del filetto'),
  writef('Per risolvere l''esercizio uso %w con profondit max. %w\n',
         [Algoritmo,MaxDepth]).

chiedi_giocatore(Num,Giocatore) :-
  writef('Tipo del giocatore %w (giocatore/computer)? ',[Num]),
  leggi(G),
  verifica_giocatore(Num,G,Giocatore).
verifica_giocatore(_,'',_) :- !, fail.
verifica_giocatore(_,Giocatore,Giocatore) :-
  giocatore_lecito(Giocatore),!.
verifica_giocatore(Num,_,Giocatore) :-
  writeln('Tipo di giocatore non valido'), 
  chiedi_giocatore(Num,Giocatore).
giocatore_lecito(computer).
giocatore_lecito(giocatore).
leggi(G) :-
  current_input(Stream),
  read_line_to_codes(Stream,Codes),
  % per evitare che la chiusura della finestra pianti la macchina
  \+ Codes = [], !, 
  name(G,Codes).

% play(+Algoritmo,+MaxDepth,+Scacchiera,+PlayerInfo,-Result)
% ----------------------------------------------------------
% ciclo principale
play(_,_,Scacchiera,PlayerInfo,Result) :-
  game_over(Scacchiera,PlayerInfo,Result),!,announce(Result).
play(Algoritmo,MaxDepth,Scacchiera,PlayerInfo,Result) :-
  scegli_mossa(Algoritmo,MaxDepth,Scacchiera,PlayerInfo,Move),
  fai_mossa(Move,Scacchiera,NewS),
  mostra_gioco(NewS,PlayerInfo),
  not_game_over(NewS,PlayerInfo,_),
  prox_giocatore(PlayerInfo,NewPlayerInfo),
  !, play(Algoritmo,MaxDepth,NewS,NewPlayerInfo,Result).
not_game_over(Scacchiera,PlayerInfo,Result) :-
  game_over(Scacchiera,PlayerInfo,Result) -> announce(Result), fail;
  true.

% prox_giocatore(+OldPlayerInfo,-NewPlayerInfo)
% ---------------------------------------------
% aggiorna giocatore
prox_giocatore((1,PlType1,PlType2),(2,PlType1,PlType2)).
prox_giocatore((2,PlType1,PlType2),(1,PlType1,PlType2)).

% nome_giocatore(+PlayerInfo,-PlayerName)
% estrae il nome del giocatore che deve giocare
nome_giocatore((NumPlayer,PlType1,PlType2),Name) :-
  PlType1 = PlType2,
  swritef(Name,'%w %w',[PlType1,NumPlayer]).
nome_giocatore((NumPlayer,PlType1,PlType2),Name) :-
  PlType1 \= PlType2,
  tipo_giocatore((NumPlayer,PlType1,PlType2),PlType),
  swritef(Name,'%w',[PlType]).

% tipo_giocatore(+PlayerInfo,-PlayerType)
% estrae il tipo del giocatore che deve giocare
tipo_giocatore((1,PlType1,_),PlType1).
tipo_giocatore((2,_,PlType2),PlType2).

% game_over(+Scacchiera,+PlayerInfo,-Result)
% ------------------------------------------
% unifica se  game over
game_over(Scacchiera,_,patta) :- piena(Scacchiera).
game_over(Scacchiera,(N,P1,P2),(N,P1,P2)) :- 
  pezzo_iniziale(Scacchiera,Pezzo),
  is_leaf((Pezzo,Scacchiera),_).
% patta e vince li metto in fondo con valuta

% announce(+Result)
% -----------------
% annuncia il risultato
announce(patta) :- writef('Partita patta').
announce(PlayerInfo) :- 
  nome_giocatore(PlayerInfo,Name),
  writef('Ha vinto il %w',[Name]).

% mostra_gioco(+Scacchiera,+PlayerInfo)
% ------------------------------------------
% mostra scacchiera e giocatore a cui tocca muovere
mostra_gioco(Scacchiera,PlayerInfo) :-
  remaining_moves(Scacchiera,RestDepth),
  Ply is 9 - RestDepth,
  writef('Gioco dopo mosse: %w\n\n',[Ply]),
  mostra_scacchiera(Scacchiera),nl,
  nome_giocatore(PlayerInfo,NomeGiocatore),
  writef('Ha mosso il %w\n\n',[NomeGiocatore]).
mostra_scacchiera(S) :-
  mostra_riga(S,S1),nl,
  writeln('-----'),
  mostra_riga(S1,S2),nl,
  writeln('-----'),
  mostra_riga(S2,_),nl.
mostra_riga(S,S1) :- 
  sub_string(S,0,1,_,C1), sub_string(S,1,1,_,C2), sub_string(S,2,1,Again,C3),
  sub_string(S,3,Again,_,S1),
  writef('%w|%w|%w',[C1,C2,C3]).
/*
mostra_scacchiera(S) :-
  mostra_riga(S,S1),nl,
  writeln('-----'),
  mostra_riga(S1,S2),nl,
  writeln('-----'),
  mostra_riga(S2,_),nl.
mostra_riga([A1,A2,A3|S1],S1) :- 
  char_code(C1,A1),char_code(C2,A2),char_code(C3,A3),
  writef('%w|%w|%w',[C1,C2,C3]).
*/

% fai_mossa(+Mossa,+VecchiaScacchiera,-NuovaScacchiera)
% -----------------------------------------------------
fai_mossa(Move,OldS,NewS) :- 
  pezzo_iniziale(OldS,Pezzo),
  move(Move,(Pezzo,OldS),(_,NewS)).

% scegli_mossa(+Algoritmo,+MaxDepth,+Scacchiera,+PlayerInfo,-Move)
% ----------------------------------------------------------------
% decide la mossa
% giocatore computer
scegli_mossa(Algoritmo,MaxDepth,Scacchiera,PlayerInfo,Move) :- 
  tipo_giocatore(PlayerInfo,computer), 
  computer_move(Algoritmo,MaxDepth,Scacchiera,Move,_).
% giocatore umano
scegli_mossa(_,_,Scacchiera,PlayerInfo,Move) :-
  tipo_giocatore(PlayerInfo,giocatore), !,
  write('Qual  la tua mossa? '),leggi_mossa(M),
  controlla_mossa(Scacchiera,PlayerInfo,M,Move).
controlla_mossa(_,_,-1,-1) :- !, fail.
controlla_mossa(Scacchiera,_,Move,Move) :-
  move((_,Scacchiera),Move), !.
controlla_mossa(Scacchiera,PlayerInfo,_,Move) :-
  writeln('Mossa sbagliata, scegli ancora (1..9)'),
  !, scegli_mossa(_,_,Scacchiera,PlayerInfo,Move).
leggi_mossa(M) :- get_single_char(I), !, name(M,[I]).
  









