% DDE server per il gioco del filetto

% carica le regole del gioco
user:file_search_path(engine,'../engine').
:- [regole].
:- [engine(minimax),
    engine(alfabeta)].

ask_server(Scacchiera,Soluzione) :-
  ask_server('minimax,9',Scacchiera,Soluzione).

% ask_server
% Pone una richiestra al server
% Service = 'prologFiletto'
% Topic = Algoritmo,Depth
% Item = Scacchiera
ask_server(Algoritmo,Scacchiera,Soluzione) :-
  open_dde_conversation(prologFiletto, Algoritmo, Handle),
  dde_request(Handle, Scacchiera, Soluzione),
  close_dde_conversation(Handle).

stop_server:-
  dde_unregister_service(prologFiletto).

% dde_register_service(+Service(+Topic,+Item,+Value),+Goal).
start_server :-
  dde_register_service(
    prologFiletto(Algoritmo,Scacchiera,Soluzione), 
    filetto(Algoritmo,Scacchiera,Soluzione)).

% filetto(+Algoritmo,+Scacchiera,-Soluzione)
% carica l'algoritmo e richiama la procedura che fa la mossa 
% Algoritmo = minimax;(minimax,Depth);alfabeta;(alfabeta,Depth)
% Scacchiera  la disposizione iniziale
% Soluzione = (Move,Value)
filetto(Topic,Item,Soluzione) :-
  writef('Received: %w,"%w"\n',[Topic,Item]),
  parse_topic(Topic,Algoritmo,MaxDepth,RequireTime),
%  carica_motore(Algoritmo),
%  name(Item,Scacchiera),
  Scacchiera = Item,
  statistics(runtime,_),
  computer_move(Algoritmo,MaxDepth,Scacchiera,Move,Value),
  statistics(runtime,[_,Time]),
  swritef(Sol,'%w,%w',[Move,Value]),
  add_time(RequireTime,Time,Sol,Soluzione),
  writef('Send: %w',[Soluzione]),nl.
add_time(no,_,Soluzione,Soluzione).
add_time(yes,Time,Sol,Soluzione) :-
  swritef(Soluzione,'%w,%w',[Sol,Time]).

parse_topic(Topic,Algoritmo,Depth,Time) :-
  concat_atom(List,',',Topic),
  separate_topic(List,Algoritmo,Depth,Time).
separate_topic([Algoritmo],Algoritmo,9,no) :- !.
separate_topic([Algoritmo,DStr],Algoritmo,Depth,no) :-
  !, name(DStr,DCode),name(Depth,DCode).
separate_topic([Algoritmo,DStr,'T'],Algoritmo,Depth,yes) :-
  separate_topic([Algoritmo,DStr],Algoritmo,Depth,no).

/*
% i due algoritmi supportati: minimax puro o con potatura alfabeta
% minimax supporta il taglio dei fratelli in caso di vittoria,
% minimax_simple  la versione pura senza taglio
% Per caricare le versioni compilate usare l'estensione (.qlf)
% comunque non c' nessun cambio di prestazioni
carica_motore(minimax) :- ensure_loaded(engine('minimax')).
carica_motore(minimax_simple) :- ensure_loaded(engine('minimax')).
carica_motore(alfabeta) :- ensure_loaded(engine('alfabeta')).
*/












