/*
Filetto
In questo file ci sono le regole del gioco richiamate 
dal motore (minimax o alfabeta)
In particolare ci sono le seguenti funzioni:
1) legal_position(+Position)
   valuta se l'input  legale
2) move(+Position,-Move) 
   genera una alla volta tutte le mosse legali
3) move(+Move,+OldPosition,-NewPosition) 
   data la mossa aggiorna la posizione
4) is_leaf(+Position, -Value) 
   funzione di valutazione per condizione di fine partita
5) euristics(+Position, -Value)
   funzione di valutazione per le situazioni restanti
6) won(+Value)
   dice se il valore della f. di valutazione  di vittoria
7) string_position(+Position,-String)
   la posizione in formato stringa (per scopi di debug)
8) string_move(+Move,-String)
   la mossa in formato stringa (per scopi di debug)
*/

%%%%%%%%%%%%%%%%
% ENTRY POINTS %
%%%%%%%%%%%%%%%%

computer_choice(Algoritmo,MaxDepth,Scacchiera,List) :-
  computer_choice(Algoritmo,MaxDepth,Scacchiera,List,none).
computer_choice(Algoritmo,MaxDepth,Scacchiera,List,Mode) :-
  computer_choice(Algoritmo,MaxDepth,Scacchiera,List,Mode,'debug.txt').
computer_choice(Algoritmo,MaxDepth,Scacchiera,List,Mode,DebugFile) :-
  calcola_depth(Scacchiera,MaxDepth,D),
  pezzo_iniziale(Scacchiera,Pezzo),
  get_all_moves(Algoritmo,D,(Pezzo,Scacchiera),List,Mode,DebugFile).
computer_move(Algoritmo,MaxDepth,Scacchiera,Move,Value) :-
  computer_move(Algoritmo,MaxDepth,Scacchiera,Move,Value,none).
computer_move(Algoritmo,MaxDepth,Scacchiera,Move,Value,Mode) :-
  computer_move(Algoritmo,MaxDepth,Scacchiera,Move,Value,Mode,'debug.txt').
computer_move(Algoritmo,MaxDepth,Scacchiera,Move,Value,Mode,DebugFile) :-
  calcola_depth(Scacchiera,MaxDepth,D),
  pezzo_iniziale(Scacchiera,Pezzo),
  choose_move(Algoritmo,D,(Pezzo,Scacchiera),Move,Value,Mode,DebugFile).

%%%%%%%%%%
% REGOLE %
%%%%%%%%%%

% legal_position(+Position)
% -------------------------
% stabilisce se la posizione  legale
% Position = (Pezzo,Scacchiera)
% Pezzo = 'X'|'O'
% Scacchiera  una lista di 9 elementi 'X'|'O'|' '
legal_position((Pezzo,Scacchiera)) :-
  pezzo_valido(Pezzo),
  string_length(Scacchiera,9),
  string_to_list(Scacchiera,List),
  caselle_lecite(List).
legal_position(_) :- write('Scacchiera illegale'),nl,fail.
caselle_lecite([]).
caselle_lecite([T|C]) :- casella_lecita(T),caselle_lecite(C).
casella_lecita(C) :-
  char_code('X',C), !; % X
  char_code('O',C), !; % O
  char_code(' ',C), !. % casella vuota
pezzo_valido('X').
pezzo_valido('O').

% move(+Position,?Move)
% ---------------------
% verifica se Move  una mossa lecita da Position
% oppure
% genera la prossima mossa legale
% RICHIAMATA ANCHE DALL'INTERFACCIA
move((_,Scacchiera),Move) :-
  sub_string(Scacchiera,M,1,_,' '),
  Move is M+1.
%  char_code(' ',Spazio), nth1(Move,Scacchiera,Spazio).

% move(+Move,+OldPosition,-NewPosition)
% -------------------------------------
% data la mossa aggiorna la posizione
% RICHIAMATA ANCHE DALL'INTERFACCIA
move(Move,(OldPezzo,OldS),(NewPezzo,NewS)) :-
  cambia_pezzo(OldPezzo,NewPezzo), !,
%  M is Move - 1, sub_string(OldS,M,1,Again,' '), !,
  L1 is Move-1,  sub_string(OldS,0,L1,_,S1),
  L2 is 9-Move, sub_string(OldS,Move,L2,_,S2),
  concat_atom([S1,NewPezzo,S2],NewS).
%  string_concat(S1,NewPezzo,S),
%  string_concat(S,S2,NewS).

% is_leaf(+Position, -Value)
% --------------------------
% funzione di valutazione
% Input: pezzo, scacchiera
% Data una posizione dice se c' una vittoria del giocatore corrente e 
% nel caso ritorna il valore della funzione di valutazione, altrimenti
% fallisce
% RICHIAMATA ANCHE DALL'INTERFACCIA
is_leaf((Pezzo,Scacchiera),Val) :-
  vince_pezzo(Pezzo,Scacchiera),!,Val is 1.

% vince_pezzo(+Pezzo,+Scacchiera) :-
% controlla se c' una vittoria del pezzo
vince_pezzo(Pezzo,Scacchiera) :-
  fa_filetto(Pezzo,Scacchiera,0,1,2);
  fa_filetto(Pezzo,Scacchiera,3,4,5);
  fa_filetto(Pezzo,Scacchiera,6,7,8);
  fa_filetto(Pezzo,Scacchiera,0,3,6);
  fa_filetto(Pezzo,Scacchiera,1,4,7);
  fa_filetto(Pezzo,Scacchiera,2,5,8);
  fa_filetto(Pezzo,Scacchiera,0,4,8);
  fa_filetto(Pezzo,Scacchiera,2,4,6).
fa_filetto(Pezzo,Scacchiera,I1,I2,I3) :-
  sub_string(Scacchiera,I1,1,_,Pezzo),
  sub_string(Scacchiera,I2,1,_,Pezzo),
  sub_string(Scacchiera,I3,1,_,Pezzo).

% euristics(+Position, -Value)
% ----------------------------
% funzione di valutazione
% Input: pezzo, scacchiera
euristics(_,0). 

% won(+Value)
% -----------
% Ha successo se  un valore di vittoria
won(1).

% string_position(+Position,-String)
string_position(Position,Position).

% string_move(+Move,-String)
string_move(Move,Move).

%%%%%%%%%
% UTILS %
%%%%%%%%%

% calcola_depth(+Scacchiera,+MaxDepth,-Depth)
calcola_depth(Scacchiera,MaxDepth,Depth) :-
  remaining_moves(Scacchiera,RestDepth),
  Depth is min(MaxDepth,RestDepth).

% pezzo_iniziale(+Scacchiera,-Pezzo)
pezzo_iniziale(Scacchiera,Pezzo) :-
  remaining_moves(Scacchiera,RestDepth),
  Ply is 9 - RestDepth,
  sceglie_pezzo(Ply,Pezzo).

% usato dall'algoritmo
cambia_pezzo('X','O').
cambia_pezzo('O','X').

% sceglie_pezzo(+Ply,-Pezzo)
% Usato dal chiamante per fornire il pezzo iniziale iniziale
sceglie_pezzo(Ply,'X') :- dispari(Ply),!.
sceglie_pezzo(_,'O').
dispari(N) :- 1 =:= N mod 2.

% remaining_moves(+Scacchiera,-RemainingMoves) 
% Usato dal chiamante per calcolare Depth
remaining_moves(Scacchiera,RestDepth) :-
  bagof(M,move((_,Scacchiera),M),Moves),
  length(Moves,RestDepth).

% piena(+Scacchiera)
% controlla se non ci sono pi mosse disponibili
% RICHIAMATA SOLO DALL'INTERFACCIA
piena(Scacchiera) :- 
  remaining_moves(Scacchiera,0).











