/*
Move/2
Genera tutte le mosse legali da una posizione una per volta

MOSSE SPECIALI
- PEDONE: all inizio muove di 1 o di 2, poi di 1
  - presa en-passant: solo quando muove di 2 pu essere mangiato da un
    pezzo affiancato (subito, non dopo)
    PROBLEMA: la prima volta viene considerato appena mosso?
  - promozione: pu diventare qualsiasi altro pezzo
    LIMITAZIONE: generiamo solo regina e cavallo
- ARROCCHI: nessuna delle caselle interessate deve essere sotto scacco,
  re e torre non devono essersi mai mossi
  - PROBLEMA: se il re e la torre sono nella loro posizione originale
    si assume che non si sono mai mossi
- CONDIZIONI DI PATTA: testare solo quella relativa a nessun movimento
  legale ( NO: patta per ripetizione e limite delle 50 mosse)
*/

:- [alfiere,cavallo,torre,re,pedone].
:- [valuate].
:- [promozione,arrocco].

% Move=(Mosso,Compagno,Trasformato,Mangiato)
move((Giocatore,Scacchiera),(Mosso,Compagno,Trasformato,Mangiato))  :-
  generaPezzi(Giocatore,Scacchiera,Tipo,Src),
  allmoves(Giocatore,Scacchiera,Tipo,Src,Mosso,Compagno,Trasformato,Mangiato),
  move((Mosso,Compagno,Trasformato,Mangiato),(Giocatore,Scacchiera),NewPos), 
  \+ scacco_re(NewPos).
 
% generaPezzi(+Giocatore,+Position,-IndicePezzo,-Src)
generaPezzi(Giocatore,(Neri,Bianchi),IndicePezzo,Src) :-
  pieces(Giocatore,(Neri,Bianchi),Pezzi,_),
  genera(Pezzi,IndicePezzo,Src).
% se abbiamo una lista vuota?
genera([(Idx,X,Y)],Idx,(X,Y)):- !.
genera([(Idx,X,Y)|_],Idx,(X,Y)).
genera([_|C],Idx,(X,Y)) :-   genera(C,Idx,(X,Y)). 

allmoves(Giocatore,Scacchiera,Tipo,Src,(Tipo,Src,Dst),nil,nil,nil) :-  
  movimento(Giocatore,Scacchiera,Tipo,Src,Dst).
allmoves(Giocatore,Scacchiera,Tipo,Src,(Tipo,Src,Dst),nil,NuovoTipo,(TipoM,Dst1)) :-  
  NuovoTipo = nil,
  presa(Giocatore,Scacchiera,Tipo,Src,Dst,Dst1,TipoM);
  promozionePresa(Giocatore,Scacchiera,Tipo,Src,NuovoTipo,Dst,TipoM).
allmoves(Giocatore,Scacchiera,Tipo,Src,(torre,Src,Dst),(re,SrcRE,DstRe),nil,nil) :-  
  arrocco(Giocatore,Scacchiera,Tipo,Src,Dst,re,SrcRE,DstRe).
allmoves(Giocatore,Scacchiera,Tipo,Src,(Tipo,Src,Dst),nil,NuovoTipo,nil) :-  
  promozione(Giocatore,Scacchiera,Tipo,Src,NuovoTipo,Dst).

allpresa(Giocatore,Scacchiera,Tipo,Src,Dst1) :-  
  presa(Giocatore,Scacchiera,Tipo,Src,_,Dst1,_);
  promozionePresa(Giocatore,Scacchiera,Tipo,Src,_,Dst1,_).

movimento(Giocatore,Scacchiera,pedone,(X0,Y0),(X0,Y)) :- 
  mossa_pedone(Giocatore,Scacchiera,(X0,Y0),(X0,Y)).
movimento(_,Scacchiera,torre,(X0,Y0),(X,Y)) :- 
   mossa_torre(Scacchiera,(X0,Y0),(X,Y)).
movimento(_,Scacchiera,cavallo,(X0,Y0),(X,Y)) :-
%  legal(X),legal(Y),
  mossa_cavallo(Scacchiera,(X0,Y0),(X,Y)).
movimento(_,Scacchiera,alfiere,(X0,Y0),(X,Y)) :-
  mossa_alfiere(Scacchiera,(X0,Y0),(X,Y)).
movimento(_,Scacchiera,regina,S,D) :-
  movimento(_,Scacchiera,alfiere,S,D); 
  movimento(_,Scacchiera,torre,S,D).
movimento(_,Scacchiera,re,(X0,Y0),(X,Y)) :-
  mossa_re(Scacchiera,(X0,Y0),(X,Y)).

%presa(Giocatore,Scacchiera,Tipo,Src,Dst,Mangiato).
% Mangiato  il tipo del pezzo che  stato mangiato

presa(Giocatore,Scacchiera,pedone,(X0,Y0),(X,Y),(X,Y),Idx) :- 
  presa_pedone(Giocatore,Scacchiera,(X0,Y0),_,Idx,(X,Y)). 

presa(Giocatore,Scacchiera,pedone,(X0,Y0),(X,Y),(X,Y),Idx) :- 
  promozionePresa(Giocatore,Scacchiera,pedone,(X0,Y0),_,(X,Y),Idx). 

presa(Giocatore,Scacchiera,pedone,(X0,Y0),(Xf,Yf),(X,Y),Idx) :- 
  presa_pedonePas(Giocatore,Scacchiera,(X0,Y0),(Xf,Yf),Idx,(X,Y)). 
  
presa(Giocatore,Scacchiera,re,(X0,Y0),(X,Y),(X,Y),Idx) :- 
  presa_re(Giocatore,Scacchiera,(X0,Y0),_,Idx,(X,Y)). 

presa(Giocatore,Scacchiera,alfiere,(X0,Y0),(X,Y),(X,Y),Idx) :- 
  presa_alfiere(Giocatore,Scacchiera,(X0,Y0),_,Idx,(X,Y)). 

presa(Giocatore,Scacchiera,cavallo,(X0,Y0),(X,Y),(X,Y),Idx) :- 
  presa_cavallo(Giocatore,Scacchiera,(X0,Y0),_,Idx,(X,Y)). 

presa(Giocatore,Scacchiera,torre,(X0,Y0),(X,Y),(X,Y),Idx) :- 
  presa_torre(Giocatore,Scacchiera,(X0,Y0),_,Idx,(X,Y)). 

presa(Giocatore,Scacchiera,regina,(X0,Y0),(X,Y),(X,Y),Idx) :- 
  presa_alfiere(Giocatore,Scacchiera,(X0,Y0),_,Idx,(X,Y));
  presa_torre(Giocatore,Scacchiera,(X0,Y0),_,Idx,(X,Y)). 














