% utils di debug

string_position((G,S),(G,SStr)) :-
  scacchiera_to_string(S,SStr).

string_move(nil,nil) :- !.
string_move(Move,MStr) :-
  mossa_to_string(Move,MStr).

% moves_to_string
moves_to_string(List,String) :-
  mosse_string(List,ListOfStrings),
  concat_atom(ListOfStrings,',',String).
mosse_string([],[]).
mosse_string([Move|Moves],[Str|L]) :-
  mossa_to_string(Move,Str),
  mosse_string(Moves,L).

% scacchiera_to_string(+List,-String)
scacchiera_to_string((N,B),String) :-
  converti_pezzi(nero,N,ListOfN),
  concat_atom(ListOfN,',',NStr),
  converti_pezzi(bianco,B,ListOfB),
  concat_atom(ListOfB,',',BStr),
  concat_atom([NStr,',',BStr],String).
converti_pezzi(_,[],[]).
converti_pezzi(Colore,[(Tipo,X,Y)|C],[NewString|L]) :-
  charTipo(Tipo,Colore,ChTipo),
  concat_atom([ChTipo,X,Y],NewString),
  converti_pezzi(Colore,C,L).

% mossa_to_string(+Mossa,-String)
mossa_to_string(((_,(X,_),_),Compagno,nil,nil),String) :-
  Compagno \= nil, !, % arrocco
  scrivi_arrocco(X,String).
mossa_to_string(((_,(SrcX,SrcY),(DstX,DstY)),nil,Trasformato,Mangiato),String) :-
  Trasformato \= nil, !, % promozione,
  charTipo(Trasformato,bianco,ChTipo),
  concat_atom(['!',ChTipo,SrcX,SrcY,DstX,DstY],S),
%  char_code('0',ChZ), ChX is ChZ+X, ChY is ChZ+Y, 
  scrivi_presa(Mangiato,S,String).
mossa_to_string(((Tipo,(SrcX,SrcY),(DstX,DstY)),nil,nil,Mangiato),String) :-
  charTipo(Tipo,bianco,ChTipo),
  concat_atom([ChTipo,SrcX,SrcY,DstX,DstY],S),
%  char_code('0',ChZ), ChX is ChZ+X, ChY is ChZ+Y, 
  scrivi_presa(Mangiato,S,String).
scrivi_arrocco(1,'o-o-o').
scrivi_arrocco(8,'o-o').
scrivi_presa(nil,S,S).
scrivi_presa((_,(X,Y)),OldStr,NewStr) :-
  concat_atom([OldStr,'x',X,Y],NewStr).

% string_to_scacchiera(+String,-Scacchiera,-Error)
string_to_scacchiera(String,Scacchiera,Error) :-
  concat_atom(List,',',String),
  scorri_lista(List,([],[]),Scacchiera,Error).
scorri_lista([],S,S,ok).
scorri_lista([T|C],OldS,NewS,E) :-
  string_to_list(T,Pezzo), 
  leggi_aggiungi_pezzo(Pezzo,OldS,S,Error),
  controllo_errore(Error,T,C,S,NewS,E).
controllo_errore(nil,_,List,OldS,NewS,E) :-
  !, scorri_lista(List,OldS,NewS,E).
controllo_errore(E,P,_,S,S,(E,P)) :-
  E = wrong_type.
controllo_errore(E,P,_,S,S,(E,P)) :-
  E = wrong_coords.
controllo_errore(E,P,_,S,S,(E,P)) :-
  E = occupied.
controllo_errore(E,P,_,S,S,(E,P)) :-
  E = wrong_length.
leggi_aggiungi_pezzo([ChTipo,ChX,ChY],OldS,NewS,nil) :-
  parse_tipo(ChTipo,Tipo,Colore),
  parse_X(ChX,X),parse_Y(ChY,Y),
  pieces(Colore,OldS,Pezzi,Altri),
  \+ memberchk((_,X,Y),Pezzi), \+ memberchk((_,X,Y),Altri), !,
%  writef('%w,%w,%w,%w\n',[Tipo,Colore,X,Y]),
  insert_piece_check(Tipo,X,Y,Pezzi,NewPezzi),
  pieces(Colore,NewS,NewPezzi,Altri).
leggi_aggiungi_pezzo([ChTipo,_,_],S,S,wrong_type) :- parse_tipo(ChTipo,errore,errore), !.
leggi_aggiungi_pezzo([_,ChX,ChY],S,S,wrong_coords) :-  
  \+ read_X(ChX,_), !;
  \+ read_Y(ChY,_), !.
leggi_aggiungi_pezzo([_,_,_],S,S,occupied) :- !.
leggi_aggiungi_pezzo(_,S,S,wrong_length).
read_Y(ChY,Y) :- parse_Y(ChY,Y), number(Y), in_range(Y).
read_X(ChX,X) :- parse_X(ChX,X),  number(X), in_range(X).
% da usarsi insieme a string_to_scacchiera
segnala_errore(ok).
segnala_errore((wrong_type,P)) :-
  writef('Errore in %w: tipo sbagliato\n',[P]).
segnala_errore((wrong_coords,P)) :-
  writef('Errore in %w: coordinate sbagliate\n',[P]).
segnala_errore((occupied,P)) :-
  writef('Errore in %w: casella gi occupata\n',[P]).
segnala_errore((wrong_length,P)) :-
  writef('Errore in %w: lunghezza sbagliata\n',[P]).

