% Utils
% Mattoni essenziali per le regole

% prendi re
re([(_,X,Y)|_],(X,Y)).

% in_range(+Coord)
% out_of_range(+Coord)
% coordinata legale/non legale?
in_range(Coord) :- Coord >= 1, Coord =< 8.
out_of_range(Coord) :- Coord < 1; Coord > 8.

% opponent(?Giocatore,?Avversario)
% restituisce l avversario
opponent(bianco,nero).
opponent(nero,bianco).

% pieces(?Colore,?Scacchiera,?Pezzi)
% separa le 2 liste di pezzi della scacchiera
% oppure
% ricompone le 2 liste in una scacchiera
pieces(bianco,(N,B),B,N).
pieces(nero,(N,B),N,B).

% player(+Player)
% giocatore legale?
player(bianco).
player(nero).

% tipo(?Tipo)
tipo(torre).
tipo(cavallo).
tipo(alfiere).
tipo(re).
tipo(regina).
tipo(pedone).

% merge_pieces(+Scacchiera,-Pezzi)
% fonde insieme le 2 liste
% ATTENZIONE: B deve essere diverso dalla lista vuota (perch?)
% perch lo dice PAola e basta! Oh
merge_pieces((B,N),U):- append(B,N,U).

% test di coordinate uguali
equal_coords((X,Y),(X,Y)).

% charTipo(?Tipo,?Colore,?Carattere) 
% ritorna il carattere associato a un tipo e a un colore
% oppure
% ritorna il tipo e il colore associato a un carattere
charTipo(torre, nero, t).
charTipo(cavallo,nero,c).
charTipo(alfiere,nero,a).
charTipo(regina,nero,d).
charTipo(re,nero,r).
charTipo(pedone,nero,p).
charTipo(torre, bianco, 'T').
charTipo(cavallo,bianco,'C').
charTipo(alfiere,bianco,'A').
charTipo(regina,bianco,'D').
charTipo(re,bianco,'R').
charTipo(pedone,bianco,'P').

% search(+S,+X,+Y,-Tipo,-Colore)
% cerca se c' un pezzo in una posizione
search((Neri,_),X,Y,Tipo,nero) :- memberchk((Tipo,X,Y),Neri),!.
search((_,Bianchi),X,Y,Tipo,bianco) :- memberchk((Tipo,X,Y),Bianchi).

% reverse_between(+I,+J,-X)
% esegue il between all'indietro
% p.e. reverse_between(8,1,X)
reverse_between(I,J,I) :- I >= J.
reverse_between(I,J,K) :- I > J, !, I1 is I-1, reverse_between(I1,J,K).

% insert_piece(+Tipo,(+X,+Y),+OldPezzi,-NewPezzi).
% inserisce un pezzo in un colore (in testa subito dopo il re)
% Nessun controllo di errore

insert_piece(re,(X,Y),[Re|Resto],[(re,X,Y),Re|Resto]) :- !.
insert_piece(Tipo,(X,Y),[Re|Resto],[Re,(Tipo,X,Y)|Resto]) :- !.
insert_piece(re,(X,Y),[],[(re,X,Y)]).
  
% remove_piece((+X,+Y),+OldPezzi,-NewPezzi).
% rimuove un pezzo da un colore
remove_piece((X,Y),[(_,X,Y)|Resto],Resto) :- !.
remove_piece((X,Y),[T|C],[T|NewPezzi]) :- remove_piece((X,Y),C,NewPezzi).

% free(+Scacchiera,+(X,Y))
free(Scacchiera,(X,Y)) :- 
  pieces(nero,Scacchiera,Neri,Bianchi), 
  \+ memberchk((_,X,Y),Neri), \+ memberchk((_,X,Y),Bianchi).
% free(+Pezzi,-Tipo,+(X,Y))
free(Pezzi,Tipo,(X,Y)) :- memberchk((Tipo,X,Y),Pezzi).

% lettura di lettera carattere e delle coordinate
parse_tipo(ChTipo,Tipo,Colore) :-
  char_code(AtomTipo,ChTipo), charTipo(Tipo,Colore,AtomTipo), !.
parse_tipo(_,errore,errore).
parse_X(ChX,X) :- char_type(ChX,digit), !, name(X,[ChX]).
parse_X(ChX,X) :- char_code(a,Cha), X is ChX - Cha + 1. 
parse_Y(ChY,Y) :- name(Y,[ChY]).

% insert_piece_check(+Tipo,+X,+Y,+OldPezzi,-NewPezzi).
% inserisce speciale (pu non esserci il re ancora)
insert_piece_check(Tipo,X,Y,[],[(Tipo,X,Y)]) :- !.
insert_piece_check(Tipo,X,Y,[(Tipo1,X1,Y1)|Resto],[(Tipo,X,Y),(Tipo1,X1,Y1)|Resto]) :-
  Tipo1 \= re, !.
insert_piece_check(Tipo,X,Y,[Re|Resto],[Re,(Tipo,X,Y)|Resto]).




