% DDE server per il gioco degli scacchi

user:file_search_path(engine,'../engine').
:- [regole].
:- [engine(minimax),
    engine(alfabeta)].

stop_server:-
  dde_unregister_service(prologFiletto).

% dde_register_service(+Service(+Topic,+Item,+Value),+Goal).
start_server :-
  dde_register_service(
    prologEngine(Topic,Item), 
    m2m(Topic,Item)).

m2m(Topic,Item) :-
  m2m(Topic,Item,Soluzione),
  spedisci_soluzione(Soluzione).

spedisci_soluzione(Stringa) :-
  write('apro il server Delphi...'), flush_output,
  open_dde_conversation('MattoIn2Mosse','DelphiTopic', Handle),
  write_ln('fatto'),
  write('spedisco il comando..'), flush_output,
  dde_execute(Handle, Stringa),
  write_ln('fatto'),
  close_dde_conversation(Handle).
  
/*
m2m(+Topic,+Pezzi,-Soluzione)
dove 
1 - Topic ha la sintassi "Algoritmo,T,D"
    dove:
    Algoritmo = alfabeta|minimax|minimax_simple
    T,D = y|n
2 - Pezzi  la lista di pezzi sulla scacchiera separati da virgole
3 - Soluzione  soluzione,[tempi],[coppie di mosse]
*/
m2m(Topic,Item,Soluzione) :-
  writef('Received: %w,"%w"',[Topic,Item]),nl,
  parse_topic(Topic,Algoritmo,RequireTime,RequireDetails),
  parse_item(Item,Scacchiera),
  writeln(Scacchiera), flush_output,
  matto2mosse(Algoritmo,Scacchiera,RequireDetails,
                Errore,Mossa,Valore,Tempi,List),
  scriveSol(Errore,Mossa,Valore,Tempi),
  prepara_soluzione(RequireTime,RequireDetails,Errore,Mossa,Tempi,List,Soluzione).  

scriveSol(Errore,Mossa,Valore,Tempi) :-
  writef('\n\nErrore: %w\nMossa: %w\nValore: %w\nTempi: %w\n\n',
         [Errore,Mossa,Valore,Tempi]).

% parse_topic
parse_topic(Topic,Algoritmo,Time,Details) :-
  concat_atom(List,',',Topic),
  separate_topic(List,Algoritmo,Time,Details).
separate_topic([Algoritmo,T,D],Algoritmo,T,D).

% parse_item
parse_item(Item,Scacchiera) :-
  string_to_scacchiera(Item,Scacchiera,Errore),
  segnala_errore(Errore).

% prepara_soluzione
prepara_soluzione(RequireTime,RequireDetails,Errore,Mossa,Tempi,List,Soluzione) :-
  Sol1 = Errore,
  add_mossa(Errore,Mossa,Sol1,Sol2),
  add_time(RequireTime,Tempi,Sol2,Sol3),
  add_details(RequireDetails,List,Sol3,Soluzione),
  writef('Send: %w',[Soluzione]),nl.
add_mossa(AllOk,Mossa,OldSol,NewSol) :- 
  all_ok(AllOk), !, mossa_to_string(Mossa,MStr),
  concat_atom([OldSol,MStr],',',NewSol).
add_mossa(_,_,S,S).
add_time(n,_,Soluzione,Soluzione) :- !.
add_time(y,Time,Sol,Soluzione) :-
  concat_atom([Sol,Time],',',Soluzione).
add_details(n,_,Soluzione,Soluzione) :- !.
add_details(y,List,Sol,Soluzione) :-
  doppie_mosse_to_string(List,String),
  concat_atom([Sol,String],',',Soluzione).
all_ok(ok).
all_ok(one_move_mate).

% doppie_mosse_to_string
doppie_mosse_to_string(List,String) :-
  doppie_mosse(List,ListOfStrings),
  concat_atom(ListOfStrings,',',String).
doppie_mosse([],[]).
doppie_mosse([(Move,CounterMove)|C],[NewString|L]) :-
  mossa_to_string(Move,S1),
  mossa_to_string(CounterMove,S2),
  concat_atom([S1,',',S2],NewString),
  doppie_mosse(C,L).
  
/*
  soluzione_spuria(Algoritmo,RequireDetails,Scacchiera,
                Errore,Mossa,Tempi,List),
*/

/*
% soluzione spuria per il debug
soluzione_spuria(_,RequireDetails,_,
                Errore,Mossa,Tempi,List) :-
  Errore = ok,
  Mossa = ((torre,(4,3),(4,4)),nil,nil,nil),
  Tempi = 743,
  chiede_dettagli(RequireDetails,List).
chiede_dettagli(n,nil).
chiede_dettagli(y,List) :-
  M1 = ((pedone,(3,4),(3,5)),nil,nil,nil),
  CM1 = ((re,(4,1),(5,2)),nil,nil,(regina,(5,2))),
  M2 = ((torre,(1,1),(4,1)),(re,(5,1),(3,1)),nil,nil),
  CM2 = ((regina,(3,4),(5,6)),nil,nil,nil),
  M3 = ((pedone,(7,7),(8,8)),nil,regina,(torre,(8,8))),
  List = [ (M1,CM1), (M2,CM2), (M3,CM1) ].
*/







