/*
Gioco degli scacchi
In questo file ci sono le regole del gioco richiamate 
dal motore (minimax o alfabeta)
In particolare ci sono le seguenti funzioni:
1) legal_position(+Position)
   valuta se l'input  legale
2) move(+Position,-Move) 
   genera una alla volta tutte le mosse legali
3) move(+Move,+OldPosition,-NewPosition) 
   data la mossa aggiorna la posizione
4) is_leaf(+Position, -Value) 
   funzione di valutazione per condizione di fine partita
5) euristics(+Position, -Value)
   funzione di valutazione per le situazioni restanti
6) won(+Value)
   dice se il valore della f. di valutazione  di vittoria
7) string_position(+Position,-String)
   la posizione in formato stringa (per scopi di debug)
8) string_move(+Move,-String)
   la mossa in formato stringa (per scopi di debug)
*/

%%%%%%%%%%
% REGOLE %
%%%%%%%%%%

:- [regole/utils].
:- [regole/legal_position,
    regole/move2,
    regole/move3,
    regole/valuate,
    regole/to_string].

%%%%%%%%%%%%%%%%
% ENTRY POINTS %
%%%%%%%%%%%%%%%%

/*
matto2mosse
Soluzione all'esercizio "il bianco matta in 2 mosse"
RequireDetails = y|n
Se RequireDetails = y fornisce i dettagli della soluzione
*/
matto2mosse(Alg,Scacchiera,RequireDetails,Errore,Mossa,Valore,Tempi,List) :-
  statistics(runtime,_),
  OldPos = (bianco,Scacchiera),
  casi_speciali(Alg,OldPos,Mossa,Valore,Errore),
  details(Errore,Alg,OldPos,RequireDetails,Mossa,Valore,List),
  statistics(runtime,[_,Tempi]).
/*
casi_speciali(Alg,Pos,Mossa,Valore,CodErrore) :-
  do_move(Alg,1,Pos,Mossa,Valore,E), won(Victory),
  ferma_se_errore_o_se(E,Valore,Victory,one_move_mate,CodErrore), !.
casi_speciali(Alg,Pos,Mossa,Valore,CodErrore) :-
  do_move(Alg,2,Pos,Mossa,Valore,E), won(Victory), Lost is -Victory,
  ferma_se_errore_o_se(E,Valore,Lost,black_mate,CodErrore), !.
*/
casi_speciali(Alg,Pos,Mossa,Valore,CodErrore) :-
  do_move(Alg,3,Pos,Mossa,Valore,E), won(Victory),
  ferma_se_errore_o_se(E,Valore,Victory,ok,CodErrore), !.
casi_speciali(_,_,_,_,no_mate).
do_move(Alg,Depth,Pos,Mossa,Valore,ok) :-
  computer_move(Alg,Depth,Pos,Mossa,Valore), !.
do_move(_,_,_,_,_,error).
ferma_se_errore_o_se(error,_,_,_,illegal_position) :- !.
ferma_se_errore_o_se(ok,Valore,Valore,CodErrore,CodErrore).
details(ok,Alg,OldPos,y,Mossa,_,List) :-
  !, move(Mossa,OldPos,NewPos),
  computer_choice(Alg,2,NewPos,L),
  subDetails(Alg,NewPos,L,List).
details(_,_,_,_,_,_,nil).
subDetails(_,_,[],[]).
subDetails(Alg,Pos,[(Mossa,_)|C],[(Mossa,ControMossa)|L]) :-
  move(Mossa,Pos,NewPos),
  computer_move(Alg,1,NewPos,ControMossa,_),
  subDetails(Alg,Pos,C,L).

% computer_choice
% fornisce la rosa delle mosse da una posizione data
computer_choice(Algoritmo,Depth,Position,List) :-
  computer_choice(Algoritmo,Depth,Position,List,none).
computer_choice(Algoritmo,Depth,Position,List,Mode) :-
  computer_choice(Algoritmo,Depth,Position,List,Mode,'debug.txt').
computer_choice(Algoritmo,Depth,(Pezzo,Scacchiera),List,Mode,DebugFile) :-
  get_all_moves(Algoritmo,Depth,(Pezzo,Scacchiera),List,Mode,DebugFile).

% computer_move
% ricerca la migliore mossa del computer
computer_move(Algoritmo,Depth,Position,Move,Value) :-
  computer_move(Algoritmo,Depth,Position,Move,Value,none).
computer_move(Algoritmo,Depth,Position,Move,Value,Mode) :-
  computer_move(Algoritmo,Depth,Position,Move,Value,Mode,'debug.txt').
computer_move(Algoritmo,Depth,(Pezzo,Scacchiera),Move,Value,Mode,DebugFile) :-
  choose_move(Algoritmo,Depth,(Pezzo,Scacchiera),Move,Value,Mode,DebugFile).












