% scacchiere di esempio

user:file_search_path(engine,'../engine').
:- [regole].
:- [engine(minimax),
    engine(alfabeta)].

% disposizioni di pezzi (esempi)
iniziale(Y1,Y2,P) :-
  P = [ (re,5,Y1),(torre,1,Y1),(cavallo,2,Y1),(alfiere,3,Y1),
        (regina,4,Y1),(alfiere,6,Y1),(cavallo,7,Y1),(torre,8,Y1),
        (pedone,1,Y2),(pedone,2,Y2), (pedone,3,Y2),(pedone,4,Y2),
        (pedone,5,Y2),(pedone,6,Y2), (pedone,7,Y2),(pedone,8,Y2) ].
pezzi(iniziale,nero,P) :- iniziale(8,7,P).
pezzi(iniziale,bianco,P) :- iniziale(1,2,P).
pezzi(1,nero, [ (re,4,8),(alfiere,5,7),(pedone,1,7),(pedone,8,7) ]).
pezzi(1,bianco, [ (re,5,1),(regina,4,1),(pedone,2,2),(pedone,4,3),
      (torre,4,4), (torre,7,3) ]).

% scacchiere (esempi)
% s(+Esempio,-Scacchiera)
s(Esempio,(Neri,Bianchi)) :- pezzi(Esempio,nero,Neri), pezzi(Esempio,bianco,Bianchi).

% edit
% oppure
% edit(+FileName)
% tool per costruire 

process :-
  unix(argv(Argv)),
  append(_PrologArgs, [--|AppArgs], Argv), !,
  scan_commands(AppArgs,CommandList),
  write_ln(CommandList),
  EmptyS = ([],[]),
  SaveFile = 's.txt',
  executeCommands(CommandList,(EmptyS,SaveFile,no),_).

% apre con scacchiera vuota
edit :-
  EmptyS = ([],[]),
  visualizza_scacchiera(EmptyS),
  editScacchiera((EmptyS,'scacchiera.txt'),_).

% carica da file scacchiera
% se lo chiamo edit sono problemi (solo da dopo un blocco macchina)
edit_file(FileName) :-
  load_scacchiera(FileName,Scacchiera),
  visualizza_scacchiera(Scacchiera),
  editScacchiera((Scacchiera,FileName),_).

editScacchiera((OldS,OldSaveFile),(NewS,NewSaveFile)) :-
  readCommands(CommandList),
  verifica_non_vuota(CommandList), !,
  executeCommands(CommandList,(OldS,OldSaveFile,no),(NewS,NewSaveFile,Scrivi)),
  visualizza_se_deve(NewS,Scrivi),
  editScacchiera((NewS,NewSaveFile,Scrivi),_).
visualizza_se_deve(_,no).
visualizza_se_deve(S,si) :- visualizza_scacchiera(S).

writeMenu(SaveFile) :-
  write_ln('Per aggiungere un pezzo: <pezzo><X><Y> (p.e. !T43 aggiunge la torre bianca in d3)'),
  write_ln('Per cancellare un pezzo: *<X><Y> (p.e. *43)'),
  write_ln('Per inserire la disposizione iniziale: i'),
  write_ln('Per svuotare la tastiera: @'),
  write_ln('Per scrivere a video la scacchiera: w'),
  write_ln('Per verificare che la scacchiera  legale: ?'),
  write_ln('Per scrivere tutte le mosse legali a video: #[C]'),
  write_ln('Per salvare su file tutte le mosse legali: #>''filename'''),
  write_ln('Per controllare se e'' scacco matto: m[C]'),
  write_ln('Per visualizzare tutte le mosse: ![C]'),
  write_ln('Per valutare la situazione corrente: v[CD]'),
  write_ln('Per caricare la scacchiera: >''filename'''),
  writef('Per salvare sull''ultima posizione: s (attualmente: ''%w'')',[SaveFile]),nl,
  write_ln('Per salvare la scacchiera su file: <''filename'''),
  write_ln('Per visualizzare questo menu: h'),
  write_ln('Per uscire: e oppure INVIO').

% readCommands(-Command)
% legge una lista di comandi seprati da virgole
readCommands(CommandList) :-
  current_input(S),
  write('Comando (h per help)? '),
  read_line_to_codes(S,Codes),
  parse_command_list(Codes,CommandList).
% parse_command_list(+Codes,-CommandList) :-
parse_command_list(Codes,CommandList) :-
  name(String,Codes),
  concat_atom(ListOfStrings,',',String),
  scan_commands(ListOfStrings,CommandList).
% scan_commands
scan_commands([],[]).
scan_commands([Str|Strings],[Com|Commands]) :-
  name(Str,Codes),
  parse_command(Codes,Com),
  scan_commands(Strings,Commands).
% parse_command(+Codes,-Command)
parse_command([Space|C],Command) :- char_code(' ',Space), !, parse_command(C,Command).
parse_command([ChTipo,ChSrcX,ChSrcY,ChDstX,ChDstY|_],
  (move,Tipo,Colore,SrcX,SrcY,DstX,DstY)) :- 
  parse_tipo(ChTipo,Tipo,Colore), Tipo \= errore, 
  parse_X(ChSrcX,SrcX),parse_Y(ChSrcY,SrcY), parse_X(ChDstX,DstX),parse_Y(ChDstY,DstY), 
  !.
parse_command([ChTipo,ChX,ChY|_],(insert,Tipo,Colore,X,Y)) :- 
  parse_tipo(ChTipo,Tipo,Colore), Tipo \= errore, 
  parse_X(ChX,X),parse_Y(ChY,Y), !.
parse_command([ChCommand,ChX,ChY|_],(remove,X,Y)) :-
  char_code('*',ChCommand), !, parse_X(ChX,X),parse_Y(ChY,Y).
parse_command([ChCommand|_],write) :-
  char_code('w',ChCommand), !.
parse_command([ChCommand|_],legal) :-
  char_code('?',ChCommand), !.
parse_command([ChCommand,ChColor|_],(mate,Colore)) :-
  char_code('m',ChCommand), scegli_colore(ChColor,Colore), !.
parse_command([ChCommand|_],(mate,bianco)) :-
  char_code('m',ChCommand), !.
parse_command([ChCommand,ChColor|_],(write_moves,Colore)) :- 
  char_code('#',ChCommand), scegli_colore(ChColor,Colore), !.
parse_command([ChCommand,ChSave|FNList],(save_moves,Filename)) :- 
  char_code('#',ChCommand), char_code('>',ChSave), name(Filename,FNList), !.
parse_command([ChCommand|_],(write_moves,bianco)) :-
  char_code('#',ChCommand), !.
parse_command([ChCommand,ChColor,ChDepth|_],(valuate,Colore,Depth)) :- 
  char_code('v',ChCommand), scegli_colore(ChColor,Colore), 
  name(Depth,[ChDepth]), number(Depth), !.
parse_command([ChCommand,ChColor|_],(valuate,Colore,3)) :- 
  char_code('v',ChCommand), scegli_colore(ChColor,Colore), !.
parse_command([ChCommand|_],(valuate,bianco,3)) :-
  char_code('v',ChCommand), !.
parse_command([ChCommand,ChColor|_],(test,Colore)) :- 
  char_code('!',ChCommand), scegli_colore(ChColor,Colore), !.
parse_command([ChCommand|_],(test,bianco)) :-
  char_code('!',ChCommand), !.
parse_command([ChCommand|_],initial) :-
  char_code('i',ChCommand), !.
parse_command([ChCommand|_],empty) :-
  char_code('@',ChCommand), !.
parse_command([ChCommand|_],help) :-
  char_code('h',ChCommand), !.
parse_command([ChCommand|FNList],(load,Filename)) :- 
  char_code('<',ChCommand), name(Filename,FNList), !.
parse_command([ChCommand|FNList],(save,Filename)) :- 
  char_code('>',ChCommand), name(Filename,FNList), !.
parse_command([ChCommand|_],save) :-
  char_code('s',ChCommand), !.
parse_command([ChCommand|_],exit) :-
  char_code('e',ChCommand), !.
parse_command(Codes,(error,Codes)).
verifica_non_vuota([]) :-
  % se la riga  vuota bisogna morire perch [] corrisponde anche alla
  % chiusura della finestra
  write_ln('riga vuota, esco'), !, fail.
verifica_non_vuota(_).
scegli_colore(ChColor,Colore) :-
  char_code(ChCol,ChColor), colore_giusto(ChCol,Colore).
colore_giusto('b',bianco).
colore_giusto('n',nero).
  
/*
% verifica_input(+OldCommand,-NewCommand)
verifica_input(error,Command) :-
  write_ln('Comando non riconosciuto'),
  !, readCommand(Command).
verifica_input(Command,Command).
*/

% executeCommands
executeCommands([],(S,File,Scrivi),(S,File,Scrivi)).
executeCommands([Com|Commands],OldConfig,NewConfig) :-
  executeCommand(Com,OldConfig,Config),
  vedi_se_continuare(Commands,Config,NewConfig).
vedi_se_continuare(_,exit,_) :- !, fail.
vedi_se_continuare(Commands,OldConfig,NewConfig) :-
  executeCommands(Commands,OldConfig,NewConfig).
% executeCommand(+Command,+OldConfig,-NewConfig)
% ----------------------------------------------
executeCommand((move,Tipo,Colore,SrcX,SrcY,DstX,DstY),(OldS,SaveFile,OldScrivi),(NewS,SaveFile,NewScrivi)) :-
  !, rimuovi_pezzo_move((SrcX,SrcY),OldS,S1,Error),
  muove(Tipo,Colore,DstX,DstY,Error,S1,NewS),
  aggiorna_scrivi(Error,OldScrivi,NewScrivi).
executeCommand((insert,Tipo,Colore,X,Y),(OldS,SaveFile,OldScrivi),(NewS,SaveFile,NewScrivi)) :-
  !, aggiungi_pezzo((Tipo,Colore,X,Y),OldS,NewS,Error),
  aggiorna_scrivi(Error,OldScrivi,NewScrivi).
executeCommand((remove,X,Y),(OldS,SaveFile,OldScrivi),(NewS,SaveFile,NewScrivi)) :-
  !, rimuovi_pezzo((X,Y),OldS,NewS,Error), 
  aggiorna_scrivi(Error,OldScrivi,NewScrivi).
executeCommand(legal,(S,SaveFile,Scrivi),(S,SaveFile,Scrivi)) :-
  !, verifica_legale(S).
executeCommand((mate,G),(S,SaveFile,Scrivi),(S,SaveFile,Scrivi)) :-
  !, controlla_matto(G,S).
executeCommand((valuate,Colore,Depth),(S,SaveFile,Scrivi),(S,SaveFile,Scrivi)) :-
  !, valuta(Colore,S,Depth).
executeCommand((write_moves,Colore),(S,SaveFile,Scrivi),(S,SaveFile,Scrivi)) :-
  !, scrivi_mosse(Colore,S).
executeCommand((save_moves,Filename),(S,SaveFile,Scrivi),(S,SaveFile,Scrivi)) :-
  !, salva_mosse(S,Filename).
executeCommand((test,Colore),(S,SaveFile,Scrivi),(S,SaveFile,Scrivi)) :-
  !, test(Colore,S).
executeCommand(help,(S,SaveFile,Scrivi),(S,SaveFile,Scrivi)) :-
  !, writeMenu(SaveFile), nl.
executeCommand(initial,(_,SaveFile,_),(NewS,SaveFile,si)) :-
  !, pezzi(iniziale,nero,Neri), pezzi(iniziale,bianco,Bianchi),  
  NewS = (Neri,Bianchi).
executeCommand(empty,(_,SaveFile,_),(NewS,SaveFile,si)) :-
  !, NewS = ([],[]).
executeCommand(write,(S,SaveFile,Scrivi),(S,SaveFile,Scrivi)) :-
  !, visualizza_scacchiera(S).
executeCommand((load,FileName),_,(NewS,FileName,si)) :-
  !, load_scacchiera(FileName,NewS).
executeCommand((save,FileName),(S,_,Scrivi),(S,FileName,Scrivi)) :-
  !, save_scacchiera(FileName,S).
executeCommand(save,(S,SaveFile,Scrivi),(S,SaveFile,Scrivi)) :-
  !, save_scacchiera(SaveFile,S).
executeCommand(exit,_,exit).
executeCommand((error,Codes),C,C) :- 
  fallisce_se_vuoto(Codes), !,
  name(Str,Codes), writef('%w: comando sconosciuto\n',[Str]).
aggiorna_scrivi(errore,S,S) :- !.
aggiorna_scrivi(_,_,si).
fallisce_se_vuoto([_|_]).

% COMANDI
% -------
% valuta
valuta(Colore,Scacchiera,Depth) :-
  write('Elaboro con alfabeta...'), flush_output,
  computer_move(alfabeta,Depth,(Colore,Scacchiera),Move,Value),
  writeln('fatto'),
  writef('Mossa: %w\nValore: %w\n',[Move,Value]).
% test
test(Giocatore,Scacchiera) :-
  writef('Generazione delle mosse del %w:\n',[Giocatore]),
  Position = (Giocatore,Scacchiera),
  verifica_legale(Scacchiera),nl,
  genera_mosse(Position,Moves),
  evaluate(Moves,Position).
evaluate([Move|Moves],Position) :-
  writef('Ora muovo: %w\n',[Move]),
  move(Move,Position,(_,NewPos)),
  visualizza_scacchiera(NewPos),
  write('INVIO per continuare, TAB per le mosse rimaste, ESC per uscire '),
  get_single_char(C), !, nl,nl,
  input_evaluate(C,Moves,Position).
evaluate([],_).
input_evaluate(-1,_,_) :- !, fail.
input_evaluate(C,_,_) :- C = 27, !.
input_evaluate(C,Moves,Position) :-
  C = 9, !, 
  writef('Mosse rimanenti:\n%w\n\n',[Moves]),
  evaluate(Moves,Position).
input_evaluate(_,Moves,Position) :-
  evaluate(Moves,Position).
% verifica_legale
verifica_legale(S) :- legal_position((bianco,S)), !, write_ln('Configurazione legale').
verifica_legale(_) :- write_ln('Configurazione non legale').
% verifica matto
controlla_matto(G,S) :- 
  scacco_matto((G,S)), !, write_ln('e'' scacco matto').
controlla_matto(_,_) :- write_ln('non e'' scacco matto').
% scrivi_mosse
scrivi_mosse(Colore,S) :- 
  genera_mosse((Colore,S),Moves),
  moves_to_string(Moves,MStr),
  writef('Mosse del %w:',[Colore]),nl,
  write_ln(MStr).
% salva_mosse
salva_mosse(S,FileName) :-
  open(FileName,write,Stream),
  writef('Salvo su %w\n\n',[FileName]),
  genera_mosse((bianco,S),WhiteMoves),
  moves_to_string(WhiteMoves,WMStr),
  writef('Mosse del bianco:\n%w\n\n',[WMStr]),
%  write(Stream,'Mosse del bianco:\n'),
  write(Stream,WMStr), nl(Stream),
  genera_mosse((nero,S),BlackMoves),
  moves_to_string(BlackMoves,BMStr),
  writef('Mosse del nero:\n%w\n\n',[BMStr]),
%  write(Stream,'Mosse del nero:\n'),
  write(Stream,BMStr), nl(Stream),
  close(Stream),
  write_ln('fatto').
% genera_mosse
genera_mosse(Position,Moves) :-
  findall(M,move(Position,M),Moves).
/*
% aggiungi_pezzi
aggiungi_pezzi([],S,S,Error,Error) :- !.
aggiungi_pezzi([(Tipo,Colore,X,Y)|Altri],OldS,NewS,OldError,NewError) :-
  aggiungi_pezzo((Tipo,Colore,X,Y),OldS,S,Error),
  aggiorna_errore(OldError,Error,NewError),
  aggiungi_pezzi(Altri,S,NewS,NewError,_).
aggiorna_errore(error,nil,nil).
aggiorna_errore(nil,error,nil).
aggiorna_errore(nil,nil,nil).
aggiorna_errore(error,error,error).
*/
aggiungi_pezzo((Tipo,Colore,X,Y),OldS,NewS,nil) :-
  pieces(Colore,OldS,Pezzi,Altri),
  coordinate_ok(X,Y),
  \+ memberchk((_,X,Y),Pezzi), \+ memberchk((_,X,Y),Altri), !,
  insert_piece_check(Tipo,X,Y,Pezzi,NewPezzi),
  pieces(Colore,NewS,NewPezzi,Altri).
aggiungi_pezzo((_,_,X,Y),S,S,error) :- 
  \+ coordinate_ok(X,Y), !,  write_ln('Errore: coordinate sbagliate').
aggiungi_pezzo(_,S,S,error) :- write_ln('Errore: casella gi occupata'),nl.
coordinate_ok(X,Y) :- in_range(X), number(Y), in_range(Y).
% rimuovi pezzo
rimuovi_pezzo((X,Y),S,S,error) :- 
  \+ coordinate_ok(X,Y), !, write_ln('Errore: coordinate sbagliate').
rimuovi_pezzo(C,OldS,NewS,nil) :- rem_piece(C,OldS,NewS), !.
rimuovi_pezzo(_,S,S,error) :- write_ln('Errore: Nessun pezzo da rimuovere'),nl.
% muove pezzo
muove(_,_,_,_,error,S,S) :- !.
muove(Tipo,Colore,DstX,DstY,_,OldS,NewS) :-
  rimuovi_pezzo((DstX,DstY),OldS,S,_),
  aggiungi_pezzo((Tipo,Colore,DstX,DstY),S,NewS,_).
% rimuovi pezzo_move
rimuovi_pezzo_move((X,Y),S,S,error) :- 
  \+ coordinate_ok(X,Y), !, write_ln('Errore: coordinate sorgente sbagliate').
rimuovi_pezzo_move(C,OldS,NewS,nil) :- rem_piece(C,OldS,NewS), !.
rimuovi_pezzo_move(_,S,S,error) :- write_ln('Errore: Nessun pezzo da muovere'),nl.
% rem_piece
rem_piece((X,Y),(Neri,Bianchi),(NewNeri,Bianchi)) :- 
  remove_piece((X,Y),Neri,NewNeri), !.
rem_piece((X,Y),(Neri,Bianchi),(Neri,NewBianchi)) :- 
  remove_piece((X,Y),Bianchi,NewBianchi), !.
% load_scacchiera
load_scacchiera(Filename,S) :- 
  writef('Carico da %w..',[Filename]),
  open(Filename,read,Stream,[eof_action(eof_code)]),
  read_stream_to_codes(Stream,Codes),
  string_to_list(String,Codes),
  string_to_scacchiera(String,S,Error),
  segnala_errore(Error),
%  name(A,Codes),
%  term_to_atom(S,A),
  close(Stream),
  write_ln('fatto').
% save_scacchiera
save_scacchiera(Filename,S) :- 
  writef('Salvo su %w..',[Filename]),
  open(Filename,write,Stream),
  scacchiera_to_string(S,String),
  write(Stream,String),
%  write(Stream,S),
  close(Stream),
  write_ln('fatto').

visualizza_scacchiera(S) :-
  nl, scriviScacchiera(S),nl,
  write_ln(S), nl.

% scriviScacchiera(+Scacchiera)
% scrive la scacchiera sul terminale
scriviScacchiera(S) :-
  write_ln('  1 2 3 4 5 6 7 8 '),
  forall(reverse_between(8,1,Num),riga(S,Num)),
  write_ln('  --------------- '),
  write_ln('  a b c d e f g h ').
riga(S,Row) :-
  write_ln('  --------------- '),
  write(Row),
  forall(between(1,8,Col),colonna(S,Col,Row)),
  write('|'),write(Row),nl.
colonna(S,Col,Row) :-
  search(S,Col,Row,Tipo,Colore) -> write('|'),charTipo(Tipo,Colore,C),write(C);
  write('| ').








